@@ -4,13 +4,13 @@ package ask
 
 import (
 	"fmt"
+	"github.com/kovidgoyal/kitty/tools/cli/markup"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/style"
+	"github.com/kovidgoyal/kitty/tools/wcswidth"
 	"io"
-	"kitty/tools/cli/markup"
-	"kitty/tools/tty"
-	"kitty/tools/tui/loop"
-	"kitty/tools/utils"
-	"kitty/tools/utils/style"
-	"kitty/tools/wcswidth"
 	"os"
 	"regexp"
 	"strings"
@@ -93,6 +93,9 @@ func GetChoices(o *Options) (response string, err error) {
 			}
 			letter = strings.ToLower(letter)
 			idx := strings.Index(strings.ToLower(text), letter)
+			if idx < 0 {
+				return "", fmt.Errorf("The choice letter %#v is not present in the choice text: %#v", letter, text)
+			}
 			idx = len([]rune(strings.ToLower(text)[:idx]))
 			allowed.Add(letter)
 			c := Choice{text: text, idx: idx, color: color, letter: letter}
@@ -127,6 +130,9 @@ func GetChoices(o *Options) (response string, err error) {
 	}
 
 	draw_long_text := func(screen_width int, text string, msg_lines []string) []string {
+		if screen_width < 3 {
+			return msg_lines
+		}
 		if text == "" {
 			msg_lines = append(msg_lines, "")
 		} else {
@@ -144,7 +150,7 @@ func GetChoices(o *Options) (response string, err error) {
 
 	ctx := style.Context{AllowEscapeCodes: true}
 
-	draw_choice_boxes := func(y, screen_width, screen_height int, choices ...Choice) {
+	draw_choice_boxes := func(y, screen_width, _ int, choices ...Choice) {
 		clickable_ranges = map[string][]Range{}
 		width := screen_width - 2
 		current_line_length := 0
@@ -396,7 +402,7 @@ func GetChoices(o *Options) (response string, err error) {
 		if ev.MatchesPressOrRepeat("esc") || ev.MatchesPressOrRepeat("ctrl+c") {
 			ev.Handled = true
 			lp.Quit(1)
-		} else if ev.MatchesPressOrRepeat("enter") {
+		} else if ev.MatchesPressOrRepeat("enter") || ev.MatchesPressOrRepeat("kp_enter") {
 			ev.Handled = true
 			response = response_on_accept
 			lp.Quit(0)
