@@ -97,29 +97,32 @@ func (m Mode) WindowTitle() string {
 }
 
 type render_state struct {
-	num_matches, num_of_slots, num_before, num_per_column, num_columns, num_shown int
-	first_idx                                                                     CollectionIndex
+	num_matches, num_of_slots, num_before, num_per_column, num_columns, num_shown, preview_width int
+	first_idx                                                                                    CollectionIndex
 }
 
 type State struct {
-	base_dir                 string
-	current_dir              string
-	select_dirs              bool
-	multiselect              bool
-	search_text              string
-	mode                     Mode
-	suggested_save_file_name string
-	suggested_save_file_path string
-	window_title             string
-	screen                   Screen
-	current_filter           string
-	filter_map               map[string]Filter
-	filter_names             []string
-	show_hidden              bool
-	respect_ignores          bool
-	sort_by_last_modified    bool
-	global_ignores           ignorefiles.IgnoreFile
-	keyboard_shortcuts       []*config.KeyAction
+	base_dir                            string
+	current_dir                         string
+	multiselect                         bool
+	search_text                         string
+	mode                                Mode
+	suggested_save_file_name            string
+	suggested_save_file_path            string
+	window_title                        string
+	screen                              Screen
+	current_filter                      string
+	filter_map                          map[string]Filter
+	filter_names                        []string
+	show_hidden                         bool
+	show_preview                        bool
+	respect_ignores                     bool
+	sort_by_last_modified               bool
+	global_ignores                      ignorefiles.IgnoreFile
+	keyboard_shortcuts                  []*config.KeyAction
+	display_title                       bool
+	pygments_style, dark_pygments_style string
+	syntax_aliases                      map[string]string
 
 	selections    []string
 	current_idx   CollectionIndex
@@ -128,13 +131,16 @@ type State struct {
 	redraw_needed bool
 }
 
+func (s State) HighlightStyles() (string, string)     { return s.pygments_style, s.dark_pygments_style }
+func (s State) SyntaxAliases() map[string]string      { return s.syntax_aliases }
+func (s State) DisplayTitle() bool                    { return s.display_title }
 func (s State) ShowHidden() bool                      { return s.show_hidden }
+func (s State) ShowPreview() bool                     { return s.show_preview }
 func (s State) RespectIgnores() bool                  { return s.respect_ignores }
 func (s State) SortByLastModified() bool              { return s.sort_by_last_modified }
 func (s State) GlobalIgnores() ignorefiles.IgnoreFile { return s.global_ignores }
 func (s State) BaseDir() string                       { return utils.IfElse(s.base_dir == "", default_cwd, s.base_dir) }
 func (s State) Filter() Filter                        { return s.filter_map[s.current_filter] }
-func (s State) SelectDirs() bool                      { return s.select_dirs }
 func (s State) Multiselect() bool                     { return s.multiselect }
 func (s State) String() string                        { return utils.Repr(s) }
 func (s State) SearchText() string                    { return s.search_text }
@@ -207,6 +213,7 @@ type Handler struct {
 	shortcut_tracker config.ShortcutTracker
 	msg_printer      *message.Printer
 	spinner          *tui.Spinner
+	preview_manager  *PreviewManager
 }
 
 func (h *Handler) draw_screen() (err error) {
@@ -225,9 +232,14 @@ func (h *Handler) draw_screen() (err error) {
 		h.lp.SetWindowTitle(h.state.WindowTitle())
 		defer func() { // so that the cursor ends up in the right place
 			h.lp.MoveCursorTo(1, 1)
-			h.draw_search_bar(0)
+			if h.state.DisplayTitle() {
+				h.lp.Println(h.state.WindowTitle())
+				h.draw_search_bar(1)
+			} else {
+				h.draw_search_bar(0)
+			}
 		}()
-		y := SEARCH_BAR_HEIGHT
+		y := SEARCH_BAR_HEIGHT + utils.IfElse(h.state.DisplayTitle(), 1, 0)
 		footer_height, err := h.draw_footer()
 		if err != nil {
 			return err
@@ -469,12 +481,19 @@ func (h *Handler) dispatch_action(name, args string) (err error) {
 		}
 	case "typename":
 		if !h.state.mode.CanSelectNonExistent() {
+			if h.state.mode.OnlyDirs() {
+				h.state.AddSelection(h.state.CurrentDir())
+				return h.finish_selection()
+			}
 			h.lp.Beep()
 		} else {
 			return h.switch_to_save_file_name_mode()
 		}
 	case "toggle":
 		switch args {
+		case "preview":
+			h.state.show_preview = !h.state.show_preview
+			return h.draw_screen()
 		case "dotfiles":
 			h.state.show_hidden = !h.state.show_hidden
 			h.result_manager.set_show_hidden()
@@ -492,9 +511,9 @@ func (h *Handler) dispatch_action(name, args string) (err error) {
 		}
 	case "cd":
 		switch args {
-		case "current":
+		case ".":
 			return h.change_to_current_dir_if_possible()
-		case "up":
+		case "..":
 			curr := h.state.CurrentDir()
 			switch curr {
 			case "/":
@@ -517,10 +536,11 @@ func (h *Handler) dispatch_action(name, args string) (err error) {
 			if absp, err := filepath.Abs(args); err == nil {
 				h.change_current_dir(absp)
 				return h.draw_screen()
+			} else {
+				h.lp.Beep()
+				return nil
 			}
-
 		}
-
 	}
 	return
 }
@@ -569,6 +589,7 @@ func (h *Handler) OnText(text string, from_key_event, in_bracketed_paste bool) (
 
 type CachedValues struct {
 	Show_hidden           bool `json:"show_hidden"`
+	Hide_preview          bool `json:"hide_preview"`
 	Respect_ignores       bool `json:"respect_ignores"`
 	Sort_by_last_modified bool `json:"sort_by_last_modified"`
 }
@@ -585,7 +606,7 @@ var cached_values = sync.OnceValue(func() *CachedValues {
 })
 
 func (s State) save_cached_values() {
-	c := CachedValues{Show_hidden: s.show_hidden, Respect_ignores: s.respect_ignores, Sort_by_last_modified: s.sort_by_last_modified}
+	c := CachedValues{Show_hidden: s.show_hidden, Respect_ignores: s.respect_ignores, Sort_by_last_modified: s.sort_by_last_modified, Hide_preview: !s.show_preview}
 	fname := filepath.Join(utils.CacheDir(), cache_filename)
 	if data, err := json.Marshal(c); err == nil {
 		_ = os.WriteFile(fname, data, 0600)
@@ -653,6 +674,8 @@ func (h *Handler) set_state_from_config(conf *Config, opts *Options) (err error)
 	h.state.sort_by_last_modified = false
 	h.state.respect_ignores = true
 	h.state.show_hidden = false
+	h.state.show_preview = true
+
 	switch conf.Show_hidden {
 	case Show_hidden_true, Show_hidden_y, Show_hidden_yes:
 		h.state.show_hidden = true
@@ -677,16 +700,29 @@ func (h *Handler) set_state_from_config(conf *Config, opts *Options) (err error)
 	case Sort_by_last_modified_last:
 		h.state.sort_by_last_modified = cached_values().Sort_by_last_modified
 	}
+	switch conf.Show_preview {
+	case Show_preview_true, Show_preview_y, Show_preview_yes:
+		h.state.show_preview = true
+	case Show_preview_false, Show_preview_n, Show_preview_no:
+		h.state.show_preview = false
+	case Show_preview_last:
+		h.state.show_preview = !cached_values().Hide_preview
+	}
+
 	h.state.global_ignores = ignorefiles.NewGitignore()
 	if err = h.state.global_ignores.LoadLines(conf.Ignore...); err != nil {
 		return err
 	}
 	h.state.keyboard_shortcuts = conf.KeyboardShortcuts
-
+	h.state.display_title = opts.DisplayTitle
+	h.state.pygments_style = conf.Pygments_style
+	h.state.dark_pygments_style = conf.Dark_pygments_style
+	h.state.syntax_aliases = conf.Syntax_aliases
 	return
 }
 
 var default_cwd string
+var use_light_colors bool
 
 func main(_ *cli.Command, opts *Options, args []string) (rc int, err error) {
 	write_output := func(selections []string, interrupted bool, current_filter string) {
@@ -737,14 +773,16 @@ func main(_ *cli.Command, opts *Options, args []string) (rc int, err error) {
 		return 1, err
 	}
 	lp.MouseTrackingMode(loop.FULL_MOUSE_TRACKING)
+	lp.ColorSchemeChangeNotifications()
 	handler := Handler{lp: lp, err_chan: make(chan error, 8), msg_printer: message.NewPrinter(utils.LanguageTag()), spinner: tui.NewSpinner("dots")}
 	handler.rl = readline.New(lp, readline.RlInit{
-		Prompt: "> ", ContinuationPrompt: ". ", Completer: handler.complete_save_prompt,
+		Prompt: "> ", ContinuationPrompt: ". ", Completer: FilePromptCompleter(handler.state.CurrentDir),
 	})
 	if err = handler.set_state_from_config(conf, opts); err != nil {
 		return 1, err
 	}
 	handler.result_manager = NewResultManager(handler.err_chan, &handler.state, lp.WakeupMainThread)
+	handler.preview_manager = NewPreviewManager(handler.err_chan, &handler.state, lp.WakeupMainThread)
 	switch len(args) {
 	case 0:
 		if default_cwd, err = os.Getwd(); err != nil {
@@ -769,12 +807,22 @@ func main(_ *cli.Command, opts *Options, args []string) (rc int, err error) {
 		if opts.Title != "" {
 			lp.SetWindowTitle(opts.Title)
 		}
+		lp.RequestCurrentColorScheme()
 		return handler.OnInitialize()
 	}
 	lp.OnResize = func(old, new_size loop.ScreenSize) (err error) {
 		handler.init_sizes(new_size)
 		return handler.draw_screen()
 	}
+	lp.OnColorSchemeChange = func(p loop.ColorPreference) (err error) {
+		new_val := p == loop.LIGHT_COLOR_PREFERENCE
+		if new_val != use_light_colors {
+			use_light_colors = new_val
+			handler.preview_manager.invalidate_color_scheme_based_cached_items()
+			return handler.draw_screen()
+		}
+		return
+	}
 	lp.OnKeyEvent = handler.OnKeyEvent
 	lp.OnText = handler.OnText
 	lp.OnMouseEvent = handler.OnMouseEvent
