@@ -9,7 +9,7 @@ import (
 	"strings"
 	"testing"
 
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils"
 
 	"github.com/google/go-cmp/cmp"
 )
@@ -19,16 +19,16 @@ var _ = fmt.Print
 func TestDiffCollectWalk(t *testing.T) {
 	tdir := t.TempDir()
 	j := func(x ...string) string { return filepath.Join(append([]string{tdir}, x...)...) }
-	os.MkdirAll(j("a", "b"), 0o700)
-	os.WriteFile(j("a/b/c"), nil, 0o600)
-	os.WriteFile(j("b"), nil, 0o600)
-	os.WriteFile(j("d"), nil, 0o600)
-	os.WriteFile(j("e"), nil, 0o600)
-	os.WriteFile(j("#d#"), nil, 0o600)
-	os.WriteFile(j("e~"), nil, 0o600)
-	os.MkdirAll(j("f"), 0o700)
-	os.WriteFile(j("f/g"), nil, 0o600)
-	os.WriteFile(j("h space"), nil, 0o600)
+	_ = os.MkdirAll(j("a", "b"), 0o700)
+	_ = os.WriteFile(j("a/b/c"), nil, 0o600)
+	_ = os.WriteFile(j("b"), nil, 0o600)
+	_ = os.WriteFile(j("d"), nil, 0o600)
+	_ = os.WriteFile(j("e"), nil, 0o600)
+	_ = os.WriteFile(j("#d#"), nil, 0o600)
+	_ = os.WriteFile(j("e~"), nil, 0o600)
+	_ = os.MkdirAll(j("f"), 0o700)
+	_ = os.WriteFile(j("f/g"), nil, 0o600)
+	_ = os.WriteFile(j("h space"), nil, 0o600)
 
 	expected_names := utils.NewSetWithItems("d", "e", "f/g", "h space")
 	expected_pmap := map[string]string{
