@@ -6,13 +6,14 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
-	"kitty/tools/utils"
-	"kitty/tools/utils/images"
-	"kitty/tools/utils/shlex"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/images"
+	"github.com/kovidgoyal/kitty/tools/utils/shlex"
 	"os/exec"
 	"path/filepath"
 	"strconv"
 	"strings"
+	"sync"
 )
 
 var _ = fmt.Print
@@ -22,11 +23,11 @@ const DIFF_DIFF = `diff -p -U _CONTEXT_ --`
 
 var diff_cmd []string
 
-var GitExe = utils.Once(func() string {
+var GitExe = sync.OnceValue(func() string {
 	return utils.FindExe("git")
 })
 
-var DiffExe = utils.Once(func() string {
+var DiffExe = sync.OnceValue(func() string {
 	return utils.FindExe("diff")
 })
 
