@@ -13,13 +13,14 @@ import kitty.fast_data_types as fast_data_types
 
 from .constants import handled_signals, is_freebsd, is_macos, kitten_exe, kitty_base_dir, shell_path, terminfo_dir
 from .types import run_once
-from .utils import cmdline_for_hold, log_error, which
+from .utils import cmdline_for_hold, log_error, resolved_shell, which
 
 if TYPE_CHECKING:
     from .window import CwdRequest
 
 
 if is_macos:
+    from kitty.fast_data_types import abspath_of_process as _abspath_of_process
     from kitty.fast_data_types import cmdline_of_process as cmdline_
     from kitty.fast_data_types import cwd_of_process as _cwd
     from kitty.fast_data_types import environ_of_process as _environ_of_process
@@ -30,6 +31,9 @@ if is_macos:
         # anyway but we use realpath for extra safety.
         return os.path.realpath(_cwd(pid))
 
+    def abspath_of_exe(pid: int) -> str:
+        return os.path.realpath(_abspath_of_process(pid))
+
     def process_group_map() -> DefaultDict[int, list[int]]:
         ans: DefaultDict[int, list[int]] = defaultdict(list)
         for pid, pgid in _process_group_map():
@@ -82,6 +86,9 @@ else:
             ans[q].append(pid)
         return ans
 
+    def abspath_of_exe(pid: int) -> str:
+        return os.path.realpath(f'/proc/{pid}/exe')
+
 
 @run_once
 def checked_terminfo_dir() -> str | None:
@@ -152,6 +159,7 @@ def process_env(env: Mapping[str, str] | None = None) -> dict[str, str]:
         ans.pop(ssl_env_var, None)
     ans.pop('XDG_ACTIVATION_TOKEN', None)
     ans.pop('VTE_VERSION', None)  # Used by the stupid VTE shell integration script that is installed system wide, sigh
+    ans.pop('KITTY_SI_RUN_COMMAND_AT_STARTUP', None)
     return ans
 
 
@@ -221,6 +229,8 @@ class Child:
         hold: bool = False,
         pass_fds: tuple[int, ...] = (),
         remote_control_fd: int = -1,
+        hold_after_ssh: bool = False,
+        startup_command_via_shell_integration: Sequence[str] | str = (),
     ):
         self.is_clone_launch = is_clone_launch
         self.id = next(child_counter)
@@ -230,7 +240,7 @@ class Child:
         self.remote_control_fd = remote_control_fd
         if cwd_from:
             try:
-                cwd = cwd_from.modify_argv_for_launch_with_cwd(self.argv, env) or cwd
+                cwd = cwd_from.modify_argv_for_launch_with_cwd(self.argv, env, hold_after_ssh=hold_after_ssh) or cwd
             except Exception as err:
                 log_error(f'Failed to read cwd of {cwd_from} with error: {err}')
         else:
@@ -238,12 +248,13 @@ class Child:
         self.cwd = os.path.abspath(cwd)
         self.stdin = stdin
         self.env = env or {}
+        self.startup_command_via_shell_integration = startup_command_via_shell_integration
         self.final_env:dict[str, str] = {}
         self.is_default_shell = bool(self.argv and self.argv[0] == shell_path)
         self.should_run_via_run_shell_kitten = is_macos and self.is_default_shell
         self.hold = hold
 
-    def get_final_env(self) -> dict[str, str]:
+    def get_final_env(self) -> tuple[dict[str, str], bool]:
         from kitty.options.utils import DELETE_ENV_VAR
         env = default_env().copy()
         opts = fast_data_types.get_options()
@@ -285,7 +296,18 @@ class Child:
             self.is_clone_launch = '1'  # free memory
         else:
             env.pop('KITTY_IS_CLONE_LAUNCH', None)
-        return env
+        must_run_startup_command_via_kitten = False
+        if self.startup_command_via_shell_integration:
+            if isinstance(self.startup_command_via_shell_integration, str):
+                env['KITTY_SI_RUN_COMMAND_AT_STARTUP'] = self.startup_command_via_shell_integration
+            else:
+                from .shell_integration import join
+                scmd = self.argv or resolved_shell(fast_data_types.get_options())
+                try:
+                    env['KITTY_SI_RUN_COMMAND_AT_STARTUP'] = join(scmd[0], self.startup_command_via_shell_integration)
+                except Exception:
+                    must_run_startup_command_via_kitten = True  # unknown shell
+        return env, must_run_startup_command_via_kitten
 
     def fork(self) -> int | None:
         if self.forked:
@@ -303,13 +325,13 @@ class Child:
             os.set_inheritable(stdin_read_fd, True)
         else:
             stdin_read_fd = stdin_write_fd = -1
-        self.final_env = self.get_final_env()
+        self.final_env, must_run_startup_command_via_kitten = self.get_final_env()
         argv = list(self.argv)
         cwd = self.cwd
         pass_fds = self.pass_fds
         if self.remote_control_fd > -1:
             pass_fds += self.remote_control_fd,
-        if self.should_run_via_run_shell_kitten:
+        if self.should_run_via_run_shell_kitten or must_run_startup_command_via_kitten:
             # bash will only source ~/.bash_profile if it detects it is a login
             # shell (see the invocation section of the bash man page), which it
             # does if argv[0] is prefixed by a hyphen see
@@ -329,6 +351,8 @@ class Child:
             if ksi == 'invalid':
                 ksi = 'enabled'
             argv = [kitten_exe(), 'run-shell', '--shell', shlex.join(argv), '--shell-integration', ksi]
+            if must_run_startup_command_via_kitten:
+                argv.extend(self.startup_command_via_shell_integration)
             if is_macos and not pass_fds and not opts.forward_stdio:
                 # In addition for getlogin() to work we need to run the shell
                 # via the /usr/bin/login wrapper, sigh.
