@@ -47,7 +47,8 @@ init_FG_BG_table(void) {
     }
 }
 
-PyObject* create_256_color_table(void) {
+static PyObject*
+create_256_color_table(void) {
     init_FG_BG_table();
     PyObject *ans = PyTuple_New(arraysz(FG_BG_256));
     if (ans == NULL) return PyErr_NoMemory();
@@ -59,21 +60,102 @@ PyObject* create_256_color_table(void) {
     return ans;
 }
 
-static PyObject *
-new_cp(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
-    ColorProfile *self;
+static void
+set_transparent_background_colors(TransparentDynamicColor *dest, PyObject *src) {
+    memset(dest, 0, sizeof(((ColorProfile*)0)->configured_transparent_colors));
+    for (Py_ssize_t i = 0; i < MIN(PyTuple_GET_SIZE(src), (Py_ssize_t)arraysz(((ColorProfile*)0)->configured_transparent_colors)); i++) {
+        PyObject *e = PyTuple_GET_ITEM(src, i);
+        dest[i].color = ((Color*)(PyTuple_GET_ITEM(e, 0)))->color.val & 0xffffff;
+        dest[i].opacity = (float)PyFloat_AsDouble(PyTuple_GET_ITEM(e, 1));
+        dest[i].is_set = true;
+    }
+}
 
+static bool
+set_configured_colors(ColorProfile *self, PyObject *opts) {
+#define n(which, attr) { \
+    RAII_PyObject(t, PyObject_GetAttrString(opts, #attr)); \
+    if (t == NULL) return false; \
+    if (t == Py_None) { self->configured.which.rgb = 0; self->configured.which.type = COLOR_IS_SPECIAL; } \
+    else if (PyLong_Check(t)) { \
+        unsigned int x = PyLong_AsUnsignedLong(t); \
+        self->configured.which.rgb = x & 0xffffff; \
+        self->configured.which.type = COLOR_IS_RGB; \
+    } else if (PyObject_TypeCheck(t, &Color_Type)) { \
+        Color *c = (Color*)t; \
+        self->configured.which.rgb = c->color.rgb; \
+        self->configured.which.type = COLOR_IS_RGB; \
+    } else { PyErr_SetString(PyExc_TypeError, "colors must be integers or Color objects"); return false; } \
+}
+
+    n(default_fg, foreground); n(default_bg, background);
+    n(cursor_color, cursor); n(cursor_text_color, cursor_text_color);
+    n(highlight_fg, selection_foreground); n(highlight_bg, selection_background);
+    n(visual_bell_color, visual_bell_color);
+#undef n
+    RAII_PyObject(src, PyObject_GetAttrString(opts, "transparent_background_colors"));
+    if (!src) { PyErr_SetString(PyExc_TypeError, "No transparent_background_colors on opts object"); return false; }
+    set_transparent_background_colors(self->configured_transparent_colors, src);
+    return PyErr_Occurred() ? false : true;
+}
+
+static bool
+set_mark_colors(ColorProfile *self, PyObject *opts) {
+    char fgattr[] = "mark?_foreground", bgattr[] = "mark?_background";
+#define n(i, attr, which) { \
+    attr[4] = '1' + i; \
+    RAII_PyObject(t, PyObject_GetAttrString(opts, attr)); \
+    if (t == NULL) return false; \
+    if (!PyObject_TypeCheck(t, &Color_Type)) { PyErr_SetString(PyExc_TypeError, "mark color is not Color object"); return false; } \
+    Color *c = (Color*)t; self->which[i] = c->color.rgb; \
+}
+#define m(i) n(i, fgattr, mark_foregrounds); n(i, bgattr, mark_backgrounds);
+    m(0); m(1); m(2);
+#undef m
+#undef n
+    return true;
+}
+
+static bool
+set_colortable(ColorProfile *self, PyObject *opts) {
+    RAII_PyObject(ct, PyObject_GetAttrString(opts, "color_table"));
+    if (!ct) return false;
+    RAII_PyObject(ret, PyObject_CallMethod(ct, "buffer_info", NULL));
+    if (!ret) return false;
+    unsigned long *color_table = PyLong_AsVoidPtr(PyTuple_GET_ITEM(ret, 0));
+    size_t count = PyLong_AsSize_t(PyTuple_GET_ITEM(ret, 1));
+    if (!color_table || count != arraysz(FG_BG_256)) { PyErr_SetString(PyExc_TypeError, "color_table has incorrect length"); return false; }
+    RAII_PyObject(r2, PyObject_GetAttrString(ct, "itemsize")); if (!r2) return false;
+    size_t itemsize = PyLong_AsSize_t(r2);
+    if (itemsize != sizeof(unsigned long)) { PyErr_Format(PyExc_TypeError, "color_table has incorrect itemsize: %zu", itemsize); return false; }
+    for (size_t i = 0; i < arraysz(FG_BG_256); i++) self->color_table[i] = color_table[i];
+    memcpy(self->orig_color_table, self->color_table, arraysz(self->color_table) * sizeof(self->color_table[0]));
+    return true;
+}
+
+
+static PyObject*
+new_cp(PyTypeObject *type, PyObject *args, PyObject *kwds) {
+    PyObject *opts = global_state.options_object;
+    ColorProfile *self;
+    static const char* kw[] = {"opts", NULL};
+    if (args && !PyArg_ParseTupleAndKeywords(args, kwds, "|O", (char**)kw, &opts)) return NULL;
     self = (ColorProfile *)type->tp_alloc(type, 0);
+    RAII_PyObject(ans, (PyObject*)self);
     if (self != NULL) {
         init_FG_BG_table();
-        memcpy(self->color_table, FG_BG_256, sizeof(FG_BG_256));
-        memcpy(self->orig_color_table, FG_BG_256, sizeof(FG_BG_256));
-#define S(which) self->mark_foregrounds[which] = OPT(mark##which##_foreground); self->mark_backgrounds[which] = OPT(mark##which##_background)
-        S(1); S(2); S(3);
-#undef S
+        if (opts) {
+            if (!set_configured_colors(self, opts)) return NULL;
+            if (!set_mark_colors(self, opts)) return NULL;
+            if (!set_colortable(self, opts)) return NULL;
+        } else {
+            memcpy(self->color_table, FG_BG_256, sizeof(FG_BG_256));
+            memcpy(self->orig_color_table, FG_BG_256, sizeof(FG_BG_256));
+        }
         self->dirty = true;
+        Py_INCREF(ans);
     }
-    return (PyObject*) self;
+    return ans;
 }
 
 static void
@@ -88,25 +170,14 @@ alloc_color_profile(void) {
 }
 
 
-static PyObject*
-update_ansi_color_table(ColorProfile *self, PyObject *val) {
-#define update_ansi_color_table_doc "Update the 256 basic colors"
-    if (!PyLong_Check(val)) { PyErr_SetString(PyExc_TypeError, "color table must be a long"); return NULL; }
-    unsigned long *color_table = PyLong_AsVoidPtr(val);
-    for (size_t i = 0; i < arraysz(FG_BG_256); i++) {
-        self->color_table[i] = color_table[i];
-        self->orig_color_table[i] = color_table[i];
-    }
-    self->dirty = true;
-    Py_RETURN_NONE;
-}
-
 void
 copy_color_profile(ColorProfile *dest, ColorProfile *src) {
     memcpy(dest->color_table, src->color_table, sizeof(dest->color_table));
     memcpy(dest->orig_color_table, src->orig_color_table, sizeof(dest->color_table));
     memcpy(&dest->configured, &src->configured, sizeof(dest->configured));
     memcpy(&dest->overridden, &src->overridden, sizeof(dest->overridden));
+    memcpy(dest->overriden_transparent_colors, src->overriden_transparent_colors, sizeof(dest->overriden_transparent_colors));
+    memcpy(dest->configured_transparent_colors, src->configured_transparent_colors, sizeof(dest->configured_transparent_colors));
     dest->dirty = true;
 }
 
@@ -138,8 +209,8 @@ patch_color_table(const char *key, PyObject *profiles, PyObject *spec, size_t wh
 
 static PyObject*
 patch_color_profiles(PyObject *module UNUSED, PyObject *args) {
-    PyObject *spec, *profiles, *v; ColorProfile *self; int change_configured;
-    if (!PyArg_ParseTuple(args, "O!O!p", &PyDict_Type, &spec, &PyTuple_Type, &profiles, &change_configured)) return NULL;
+    PyObject *spec, *transparent_background_colors, *profiles, *v; ColorProfile *self; int change_configured;
+    if (!PyArg_ParseTuple(args, "O!O!O!p", &PyDict_Type, &spec, &PyTuple_Type, &transparent_background_colors, &PyTuple_Type, &profiles, &change_configured)) return NULL;
     char key[32] = {0};
     for (size_t i = 0; i < arraysz(FG_BG_256); i++) {
         snprintf(key, sizeof(key) - 1, "color%zu", i);
@@ -173,11 +244,35 @@ patch_color_profiles(PyObject *module UNUSED, PyObject *args) {
         S(cursor_text_color, cursor_text_color); S(visual_bell_color, visual_bell_color);
 #undef SI
 #undef S
+    for (Py_ssize_t i = 0; i < PyTuple_GET_SIZE(profiles); i++) {
+        self = (ColorProfile*)PyTuple_GET_ITEM(profiles, i);
+        set_transparent_background_colors(self->overriden_transparent_colors, transparent_background_colors);
+        if (change_configured) set_transparent_background_colors(self->configured_transparent_colors, transparent_background_colors);
+    }
+    if (PyErr_Occurred()) return NULL;
     Py_RETURN_NONE;
 }
 
+bool
+colorprofile_to_transparent_color(const ColorProfile *self, unsigned index, color_type *color, float *opacity) {
+    *color = UINT32_MAX; *opacity = 1.0;
+    if (index < arraysz(self->configured_transparent_colors)) {
+        if (self->overriden_transparent_colors[index].is_set) {
+            *color = self->overriden_transparent_colors[index].color; *opacity = self->overriden_transparent_colors[index].opacity;
+            if (*opacity < 0) *opacity = OPT(background_opacity);
+            return true;
+        }
+        if (self->configured_transparent_colors[index].is_set) {
+            *color = self->configured_transparent_colors[index].color; *opacity = self->configured_transparent_colors[index].opacity;
+            if (*opacity < 0) *opacity = OPT(background_opacity);
+            return true;
+        }
+    }
+    return false;
+}
+
 DynamicColor
-colorprofile_to_color(ColorProfile *self, DynamicColor entry, DynamicColor defval) {
+colorprofile_to_color(const ColorProfile *self, DynamicColor entry, DynamicColor defval) {
     switch(entry.type) {
         case COLOR_NOT_SET:
             return defval;
@@ -208,41 +303,81 @@ colorprofile_to_color_with_fallback(ColorProfile *self, DynamicColor entry, Dyna
     }
     return entry.rgb;
 }
+static Color* alloc_color(unsigned char r, unsigned char g, unsigned char b, unsigned a);
+
+static bool
+colortable_colors_into_dict(ColorProfile *self, unsigned start, unsigned limit, PyObject *ans) {
+    static char buf[32] = {'c', 'o', 'l', 'o', 'r', 0};
+    for (unsigned i = start; i < limit; i++) {
+        snprintf(buf + 5, sizeof(buf) - 6, "%u", i);
+        PyObject *val = PyLong_FromUnsignedLong(self->color_table[i]);
+        if (!val) return false;
+        int ret = PyDict_SetItemString(ans, buf, val);
+        Py_DECREF(val);
+        if (ret != 0) return false;
+    }
+    return true;
+}
+
+static PyObject*
+basic_colors(ColorProfile *self, PyObject *args UNUSED) {
+#define basic_colors_doc "Return the basic colors as a dictionary of color_name to integer or None (names are the same as used in kitty.conf)"
+    RAII_PyObject(ans, PyDict_New()); if (ans == NULL) return NULL;
+    if (!colortable_colors_into_dict(self, 0, 16, ans)) return NULL;
+
+#define D(attr, name) { \
+    unsigned long c = colorprofile_to_color(self, self->overridden.attr, self->configured.attr).rgb; \
+    PyObject *val = PyLong_FromUnsignedLong(c); if (!val) return NULL; \
+    int ret = PyDict_SetItemString(ans, #name, val); Py_DECREF(val); \
+    if (ret != 0) return NULL; \
+}
+
+    D(default_fg, foreground); D(default_bg, background);
+#undef D
+    return Py_NewRef(ans);
+}
 
 static PyObject*
 as_dict(ColorProfile *self, PyObject *args UNUSED) {
 #define as_dict_doc "Return all colors as a dictionary of color_name to integer or None (names are the same as used in kitty.conf)"
-    PyObject *ans = PyDict_New();
-    if (ans == NULL) return PyErr_NoMemory();
-    for (unsigned i = 0; i < arraysz(self->color_table); i++) {
-        static char buf[32] = {0};
-        snprintf(buf, sizeof(buf) - 1, "color%u", i);
-        PyObject *val = PyLong_FromUnsignedLong(self->color_table[i]);
-        if (!val) { Py_CLEAR(ans); return PyErr_NoMemory(); }
-        int ret = PyDict_SetItemString(ans, buf, val);
-        Py_CLEAR(val);
-        if (ret != 0) { Py_CLEAR(ans); return NULL; }
-    }
+    RAII_PyObject(ans, PyDict_New()); if (ans == NULL) return NULL;
+    if (!colortable_colors_into_dict(self, 0, arraysz(self->color_table), ans)) return NULL;
 #define D(attr, name) { \
     if (self->overridden.attr.type != COLOR_NOT_SET) { \
         int ret; PyObject *val; \
         if (self->overridden.attr.type == COLOR_IS_SPECIAL) { \
-            val = Py_None; Py_INCREF(val); \
+            val = Py_NewRef(Py_None); \
         } else { \
-            color_type c = colorprofile_to_color(self, self->overridden.attr, self->configured.attr).rgb; \
+            unsigned long c = colorprofile_to_color(self, self->overridden.attr, self->configured.attr).rgb; \
             val = PyLong_FromUnsignedLong(c); \
         } \
-        if (!val) { Py_CLEAR(ans); return NULL; } \
+        if (!val) { return NULL; } \
         ret = PyDict_SetItemString(ans, #name, val); \
-        Py_CLEAR(val); \
-        if (ret != 0) { Py_CLEAR(ans); return NULL; } \
+        Py_DECREF(val); \
+        if (ret != 0) { return NULL; } \
     }}
     D(default_fg, foreground); D(default_bg, background);
     D(cursor_color, cursor); D(cursor_text_color, cursor_text); D(highlight_fg, selection_foreground);
     D(highlight_bg, selection_background); D(visual_bell_color, visual_bell_color);
-
+    RAII_PyObject(transparent_background_colors, PyList_New(0));
+    if (!transparent_background_colors) return NULL;
+    for (size_t i = 0; i < arraysz(self->overriden_transparent_colors); i++) {
+        TransparentDynamicColor *c = NULL;
+        if (self->overriden_transparent_colors[i].is_set) c = self->overriden_transparent_colors + i;
+        else if (self->configured_transparent_colors[i].is_set) c = self->configured_transparent_colors + i;
+        if (c) {
+            RAII_PyObject(t, Py_BuildValue("Nf", alloc_color((c->color >> 16) & 0xff, (c->color >> 8) & 0xff, c->color & 0xff, 0), c->opacity));
+            if (!t) return NULL;
+            if (PyList_Append(transparent_background_colors, t) != 0) return NULL;
+        }
+    }
+    if (PyList_GET_SIZE(transparent_background_colors)) {
+        RAII_PyObject(t, PyList_AsTuple(transparent_background_colors));
+        if (!t) return NULL;
+        if (PyDict_SetItemString(ans, "transparent_background_colors", t) != 0) return NULL;
+    }
 #undef D
-    return ans;
+    return Py_NewRef(ans);
 }
 
 static PyObject*
@@ -300,21 +435,6 @@ set_color(ColorProfile *self, PyObject *args) {
     Py_RETURN_NONE;
 }
 
-static PyObject*
-set_configured_colors(ColorProfile *self, PyObject *args) {
-#define set_configured_colors_doc "Set the configured colors"
-    unsigned int default_fg, default_bg, cursor_color, cursor_text_color, highlight_fg, highlight_bg, visual_bell_color;
-    if (!PyArg_ParseTuple(args, "II|IIIII", &default_fg, &default_bg,
-        &cursor_color, &cursor_text_color, &highlight_fg, &highlight_bg, &visual_bell_color)) return NULL;
-#define S(which) \
-    self->configured.which.rgb = which & 0xffffff; \
-    self->configured.which.type = (which & 0xff000000) ? COLOR_IS_RGB : COLOR_IS_SPECIAL;
-    S(default_fg); S(default_bg); S(cursor_color); S(cursor_text_color); S(highlight_fg); S(highlight_bg); S(visual_bell_color);
-#undef S
-    self->dirty = true;
-    Py_RETURN_NONE;
-}
-
 void
 copy_color_table_to_buffer(ColorProfile *self, color_type *buf, int offset, size_t stride) {
     size_t i;
@@ -332,6 +452,8 @@ copy_color_table_to_buffer(ColorProfile *self, color_type *buf, int offset, size
 
 static void
 push_onto_color_stack_at(ColorProfile *self, unsigned int i) {
+    self->color_stack[i].dynamic_colors = self->overridden;
+    memcpy(self->color_stack[i].transparent_colors, self->overriden_transparent_colors, sizeof(self->overriden_transparent_colors));
     self->color_stack[i].dynamic_colors = self->overridden;
     memcpy(self->color_stack[i].color_table, self->color_table, sizeof(self->color_stack->color_table));
 }
@@ -340,6 +462,7 @@ static void
 copy_from_color_stack_at(ColorProfile *self, unsigned int i) {
     self->overridden = self->color_stack[i].dynamic_colors;
     memcpy(self->color_table, self->color_stack[i].color_table, sizeof(self->color_table));
+    memcpy(self->overriden_transparent_colors, self->color_stack[i].transparent_colors, sizeof(self->overriden_transparent_colors));
 }
 
 bool
@@ -404,23 +527,37 @@ default_color_table(PyObject *self UNUSED, PyObject *args UNUSED) {
 
 // Boilerplate {{{
 
-#define CGETSET(name) \
-    static PyObject* name##_get(ColorProfile *self, void UNUSED *closure) { return PyLong_FromUnsignedLong(colorprofile_to_color(self, self->overridden.name, self->configured.name).rgb);  } \
+#define CGETSET(name, nullable) \
+    static PyObject* name##_get(ColorProfile *self, void UNUSED *closure) {  \
+        DynamicColor ans = colorprofile_to_color(self, self->overridden.name, self->configured.name);  \
+        if (ans.type == COLOR_IS_SPECIAL) { \
+            if (nullable) Py_RETURN_NONE; \
+            return (PyObject*)alloc_color(0, 0, 0, 0); \
+        } \
+        return (PyObject*)alloc_color((ans.rgb >> 16) & 0xff, (ans.rgb >> 8) & 0xff, ans.rgb & 0xff, 0); \
+    } \
     static int name##_set(ColorProfile *self, PyObject *v, void UNUSED *closure) { \
-        if (v == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute: " #name); return -1; } \
-        unsigned long val = PyLong_AsUnsignedLong(v); \
-        self->overridden.name.rgb = val & 0xffffff; \
-        self->overridden.name.type = (val & 0xff000000) ? COLOR_IS_RGB : COLOR_NOT_SET; \
+        if (v == NULL) { self->overridden.name.val = 0; return 0; } \
+        if (PyLong_Check(v)) { \
+            unsigned long val = PyLong_AsUnsignedLong(v); \
+            self->overridden.name.rgb = val & 0xffffff; \
+            self->overridden.name.type = COLOR_IS_RGB; \
+        } else if (PyObject_TypeCheck(v, &Color_Type)) { \
+            Color *c = (Color*)v; self->overridden.name.rgb = c->color.rgb; self->overridden.name.type = COLOR_IS_RGB; \
+        } else if (v == Py_None) { \
+            if (!nullable) { PyErr_SetString(PyExc_TypeError, #name " cannot be set to None"); return -1; } \
+            self->overridden.name.type = COLOR_IS_SPECIAL; self->overridden.name.rgb = 0; \
+        } \
         self->dirty = true; return 0; \
     }
 
-CGETSET(default_fg)
-CGETSET(default_bg)
-CGETSET(cursor_color)
-CGETSET(cursor_text_color)
-CGETSET(highlight_fg)
-CGETSET(highlight_bg)
-CGETSET(visual_bell_color)
+CGETSET(default_fg, false)
+CGETSET(default_bg, false)
+CGETSET(cursor_color, true)
+CGETSET(cursor_text_color, true)
+CGETSET(highlight_fg, true)
+CGETSET(highlight_bg, true)
+CGETSET(visual_bell_color, true)
 #undef CGETSET
 
 static PyGetSetDef cp_getsetters[] = {
@@ -439,15 +576,58 @@ static PyMemberDef cp_members[] = {
     {NULL}
 };
 
+static PyObject*
+reload_from_opts(ColorProfile *self, PyObject *args UNUSED) {
+    PyObject *opts = global_state.options_object;
+    if (!PyArg_ParseTuple(args, "|O", &opts)) return NULL;
+    self->dirty = true;
+    if (!set_configured_colors(self, opts)) return NULL;
+    if (!set_mark_colors(self, opts)) return NULL;
+    if (!set_colortable(self, opts)) return NULL;
+    Py_RETURN_NONE;
+}
+
+static PyObject*
+get_transparent_background_color(ColorProfile *self, PyObject *index) {
+    if (!PyLong_Check(index)) { PyErr_SetString(PyExc_TypeError, "index must be an int"); return NULL; }
+    unsigned long idx = PyLong_AsUnsignedLong(index);
+    if (PyErr_Occurred()) return NULL;
+    if (idx >= arraysz(self->configured_transparent_colors)) Py_RETURN_NONE;
+    TransparentDynamicColor *c = self->overriden_transparent_colors[idx].is_set ? self->overriden_transparent_colors + idx : self->configured_transparent_colors + idx;
+    if (!c->is_set) Py_RETURN_NONE;
+    float opacity = c->opacity >= 0 ? c->opacity : OPT(background_opacity);
+    return (PyObject*)alloc_color((c->color >> 16) & 0xff, (c->color >> 8) & 0xff, c->color & 0xff, (unsigned)(255.f * opacity));
+}
+
+static PyObject*
+set_transparent_background_color(ColorProfile *self, PyObject *const *args, Py_ssize_t nargs) {
+    if (nargs < 1) { PyErr_SetString(PyExc_TypeError, "must specify index"); return NULL; }
+    if (!PyLong_Check(args[0])) { PyErr_SetString(PyExc_TypeError, "index must be an int"); return NULL; }
+    unsigned long idx = PyLong_AsUnsignedLong(args[0]);
+    if (PyErr_Occurred()) return NULL;
+    if (idx >= arraysz(self->configured_transparent_colors)) Py_RETURN_NONE;
+    if (nargs < 2) { self->overriden_transparent_colors[idx].is_set = false; Py_RETURN_NONE; }
+    if (!PyObject_TypeCheck(args[1], &Color_Type)) { PyErr_SetString(PyExc_TypeError, "color must be Color object"); return NULL; }
+    Color *c = (Color*)args[1];
+    float opacity = (float)(c->color.alpha) / 255.f;
+    if (nargs > 2 && PyFloat_Check(args[2])) opacity = (float)PyFloat_AsDouble(args[2]);
+    self->overriden_transparent_colors[idx].is_set = true;
+    self->overriden_transparent_colors[idx].color = c->color.rgb;
+    self->overriden_transparent_colors[idx].opacity = MAX(-1.f, MIN(opacity, 1.f));
+    Py_RETURN_NONE;
+}
+
 static PyMethodDef cp_methods[] = {
-    METHOD(update_ansi_color_table, METH_O)
     METHOD(reset_color_table, METH_NOARGS)
     METHOD(as_dict, METH_NOARGS)
+    METHOD(basic_colors, METH_NOARGS)
     METHOD(color_table_address, METH_NOARGS)
     METHOD(as_color, METH_O)
     METHOD(reset_color, METH_O)
     METHOD(set_color, METH_VARARGS)
-    METHOD(set_configured_colors, METH_VARARGS)
+    METHODB(get_transparent_background_color, METH_O),
+    METHODB(reload_from_opts, METH_VARARGS),
+    {"set_transparent_background_color", (PyCFunction)(void(*)(void))set_transparent_background_color, METH_FASTCALL, ""},
     {NULL}  /* Sentinel */
 };
 
@@ -466,29 +646,31 @@ PyTypeObject ColorProfile_Type = {
 };
 // }}}
 
-
-static PyObject *
-new_color(PyTypeObject *type, PyObject *args, PyObject *kwds) {
-    static const char* kwlist[] = {"red", "green", "blue", "alpha", NULL};
-    Color *self;
-    unsigned char r = 0, g = 0, b = 0, a = 0;
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|BBBB", (char**)kwlist, &r, &g, &b, &a)) return NULL;
-
-    self = (Color *)type->tp_alloc(type, 0);
+static Color*
+alloc_color(unsigned char r, unsigned char g, unsigned char b, unsigned a) {
+    Color *self = (Color *)(&Color_Type)->tp_alloc(&Color_Type, 0);
     if (self != NULL) {
         self->color.r = r; self->color.g = g; self->color.b = b; self->color.a = a;
     }
-    return (PyObject*) self;
+    return self;
+}
+
+static PyObject *
+new_color(PyTypeObject *type UNUSED, PyObject *args, PyObject *kwds) {
+    static const char* kwlist[] = {"red", "green", "blue", "alpha", NULL};
+    unsigned char r = 0, g = 0, b = 0, a = 0;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|BBBB", (char**)kwlist, &r, &g, &b, &a)) return NULL;
+    return (PyObject*) alloc_color(r, g, b, a);
 }
 
 static PyObject*
-color_as_int(Color *self) {
+Color_as_int(Color *self) {
     return PyLong_FromUnsignedLong(self->color.val);
 }
 
 static PyObject*
 color_truediv(Color *self, PyObject *divisor) {
-    DECREF_AFTER_FUNCTION PyObject *o = PyNumber_Float(divisor);
+    RAII_PyObject(o, PyNumber_Float(divisor));
     if (o == NULL) return NULL;
     double r = self->color.r, g = self->color.g, b = self->color.b, a = self->color.a;
     double d = PyFloat_AS_DOUBLE(o) * 255.;
@@ -496,7 +678,7 @@ color_truediv(Color *self, PyObject *divisor) {
 }
 
 static PyNumberMethods color_number_methods = {
-    .nb_int = (unaryfunc)color_as_int,
+    .nb_int = (unaryfunc)Color_as_int,
     .nb_true_divide = (binaryfunc)color_truediv,
 };
 
@@ -515,7 +697,13 @@ rgb_get(Color *self, void *closure UNUSED) {
 
 static PyObject*
 luminance_get(Color *self, void *closure UNUSED) {
-    return PyFloat_FromDouble(rgb_luminance(self->color));
+    return PyFloat_FromDouble(rgb_luminance(self->color) / 255.0);
+}
+
+static PyObject*
+is_dark_get(Color *self, void *closure UNUSED) {
+    if (rgb_luminance(self->color) / 255.0 < 0.5) Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
 }
 
 static PyObject*
@@ -563,6 +751,7 @@ static PyGetSetDef color_getsetters[] = {
     {"luminance", (getter) luminance_get, NULL, "luminance", NULL},
     {"as_sgr", (getter) sgr_get, NULL, "as_sgr", NULL},
     {"as_sharp", (getter) sharp_get, NULL, "as_sharp", NULL},
+    {"is_dark", (getter) is_dark_get, NULL, "is_dark", NULL},
     {NULL}  /* Sentinel */
 };
 
@@ -612,7 +801,6 @@ static PyMethodDef module_methods[] = {
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
-
 int init_ColorProfile(PyObject *module) {\
     if (PyType_Ready(&ColorProfile_Type) < 0) return 0;
     if (PyModule_AddObject(module, "ColorProfile", (PyObject *)&ColorProfile_Type) != 0) return 0;
