@@ -5,12 +5,12 @@
  * Distributed under terms of the GPL3 license.
  */
 
-#include "data-types.h"
+#include "line.h"
 
 #include <structmember.h>
 
 static PyObject *
-new(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
+new_cursor_object(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
     Cursor *self;
 
     self = (Cursor *)type->tp_alloc(type, 0);
@@ -24,25 +24,25 @@ dealloc(Cursor* self) {
 
 #define EQ(x) (a->x == b->x)
 static int __eq__(Cursor *a, Cursor *b) {
-    return EQ(bold) && EQ(italic) && EQ(strikethrough) && EQ(dim) && EQ(reverse) && EQ(decoration) && EQ(fg) && EQ(bg) && EQ(decoration_fg) && EQ(x) && EQ(y) && EQ(shape) && EQ(non_blinking);
+    return memcmp(&a->sgr, &b->sgr, sizeof(a->sgr)) == 0 && EQ(x) && EQ(y) && EQ(shape) && EQ(non_blinking);
 }
+#undef EQ
 
-static const char* cursor_names[NUM_OF_CURSOR_SHAPES] = { "NO_SHAPE", "BLOCK", "BEAM", "UNDERLINE" };
+static const char* cursor_names[NUM_OF_CURSOR_SHAPES] = { "NO_SHAPE", "BLOCK", "BEAM", "UNDERLINE", "HOLLOW" };
 
 #define BOOL(x) ((x) ? Py_True : Py_False)
 static PyObject *
 repr(Cursor *self) {
     return PyUnicode_FromFormat(
-        "Cursor(x=%u, y=%u, shape=%s, blink=%R, fg=#%08x, bg=#%08x, bold=%R, italic=%R, reverse=%R, strikethrough=%R, dim=%R, decoration=%d, decoration_fg=#%08x)",
+        "Cursor(x=%u, y=%u, shape=%s, blink=%R, fg=#%08x, bg=#%08x, bold=%R, italic=%R, reverse=%R, strikethrough=%R, dim=%R, decoration=%d, decoration_fg=#%08x, text_blink=%R)",
         self->x, self->y, (self->shape < NUM_OF_CURSOR_SHAPES ? cursor_names[self->shape] : "INVALID"),
-        BOOL(!self->non_blinking), self->fg, self->bg, BOOL(self->bold), BOOL(self->italic), BOOL(self->reverse), BOOL(self->strikethrough), BOOL(self->dim), self->decoration, self->decoration_fg
+        BOOL(!self->non_blinking), self->sgr.fg, self->sgr.bg, BOOL(self->sgr.bold), BOOL(self->sgr.italic), BOOL(self->sgr.reverse), BOOL(self->sgr.strikethrough), BOOL(self->sgr.dim), self->sgr.decoration, self->sgr.decoration_fg, BOOL(self->sgr.blink)
     );
 }
 
 void
 cursor_reset_display_attrs(Cursor *self) {
-    self->bg = 0; self->fg = 0; self->decoration_fg = 0;
-    self->decoration = 0; self->bold = false; self->italic = false; self->reverse = false; self->strikethrough = false; self->dim = false;
+    zero_at_ptr(&self->sgr);
 }
 
 
@@ -57,10 +57,10 @@ parse_color(int *params, unsigned int *i, unsigned int count, uint32_t *result)
                 if (*i < count) *result = (params[(*i)++] & 0xFF) << 8 | 1;
                 break;
             case 2: \
-                if (*i < count - 2) {
+                if (*i + 2 < count) {
                     /* Ignore the first parameter in a four parameter RGB */
                     /* sequence (unused color space id), see https://github.com/kovidgoyal/kitty/issues/227 */
-                    if (*i < count - 3) (*i)++;
+                    if (*i +3 < count) (*i)++;
                     r = params[(*i)++] & 0xFF;
                     g = params[(*i)++] & 0xFF;
                     b = params[(*i)++] & 0xFF;
@@ -73,7 +73,7 @@ parse_color(int *params, unsigned int *i, unsigned int count, uint32_t *result)
 
 
 void
-cursor_from_sgr(Cursor *self, int *params, unsigned int count) {
+cursor_from_sgr(Cursor *self, int *params, unsigned int count, bool is_group) {
 #define SET_COLOR(which) { parse_color(params, &i, count, &self->which); } break;
 START_ALLOW_CASE_RANGE
     unsigned int i = 0, attr;
@@ -84,59 +84,68 @@ START_ALLOW_CASE_RANGE
             case 0:
                 cursor_reset_display_attrs(self);  break;
             case 1:
-                self->bold = true;  break;
+                self->sgr.bold = true;  break;
             case 2:
-                self->dim = true; break;
+                self->sgr.dim = true; break;
             case 3:
-                self->italic = true;  break;
+                self->sgr.italic = true;  break;
             case 4:
-                if (i < count) { self->decoration = MIN(5, params[i]); i++; }
-                else self->decoration = 1;
+                if (is_group && i < count) { self->sgr.decoration = MIN(5, params[i]); i++; }
+                else self->sgr.decoration = 1;
                 break;
+            case 5:
+                self->sgr.blink = true; break;
             case 7:
-                self->reverse = true;  break;
+                self->sgr.reverse = true;  break;
             case 9:
-                self->strikethrough = true;  break;
+                self->sgr.strikethrough = true;  break;
             case 21:
-                self->decoration = 2; break;
+                self->sgr.decoration = 2; break;
+            case 221:
+                self->sgr.bold = false; break;
+            case 222:
+                self->sgr.dim = false; break;
             case 22:
-                self->bold = false;  self->dim = false; break;
+                self->sgr.bold = false;  self->sgr.dim = false; break;
             case 23:
-                self->italic = false;  break;
+                self->sgr.italic = false;  break;
             case 24:
-                self->decoration = 0;  break;
+                self->sgr.decoration = 0;  break;
+            case 25:
+                self->sgr.blink = false; break;
             case 27:
-                self->reverse = false;  break;
+                self->sgr.reverse = false;  break;
             case 29:
-                self->strikethrough = false;  break;
+                self->sgr.strikethrough = false;  break;
             case 30 ... 37:
-                self->fg = ((attr - 30) << 8) | 1;  break;
+                self->sgr.fg = ((attr - 30) << 8) | 1;  break;
             case 38:
-                SET_COLOR(fg);
+                SET_COLOR(sgr.fg);
             case 39:
-                self->fg = 0;  break;
+                self->sgr.fg = 0;  break;
             case 40 ... 47:
-                self->bg = ((attr - 40) << 8) | 1;  break;
+                self->sgr.bg = ((attr - 40) << 8) | 1;  break;
             case 48:
-                SET_COLOR(bg);
+                SET_COLOR(sgr.bg);
             case 49:
-                self->bg = 0;  break;
+                self->sgr.bg = 0;  break;
             case 90 ... 97:
-                self->fg = ((attr - 90 + 8) << 8) | 1;  break;
+                self->sgr.fg = ((attr - 90 + 8) << 8) | 1;  break;
             case 100 ... 107:
-                self->bg = ((attr - 100 + 8) << 8) | 1;  break;
+                self->sgr.bg = ((attr - 100 + 8) << 8) | 1;  break;
             case DECORATION_FG_CODE:
-                SET_COLOR(decoration_fg);
+                SET_COLOR(sgr.decoration_fg);
             case DECORATION_FG_CODE + 1:
-                self->decoration_fg = 0; break;
+                self->sgr.decoration_fg = 0; break;
         }
+        if (is_group) break;
     }
 #undef SET_COLOR
 END_ALLOW_CASE_RANGE
 }
 
 void
-apply_sgr_to_cells(GPUCell *first_cell, unsigned int cell_count, int *params, unsigned int count) {
+apply_sgr_to_cells(GPUCell *first_cell, unsigned int cell_count, int *params, unsigned int count, bool is_group) {
 #define RANGE for(unsigned c = 0; c < cell_count; c++, cell++)
 #define SET_COLOR(which) { color_type color = 0; parse_color(params, &i, count, &color); if (color) { RANGE { cell->which = color; }} } break;
 #define SIMPLE(which, val) RANGE { cell->which = (val); } break;
@@ -161,21 +170,29 @@ apply_sgr_to_cells(GPUCell *first_cell, unsigned int cell_count, int *params, un
                 S(italic, true);
             case 4: {
                 uint8_t val = 1;
-                if (i < count) { val = MIN(5, params[i]); i++; }
+                if (is_group && i < count) { val = MIN(5, params[i]); i++; }
                 S(decoration, val);
             }
+            case 5:
+                S(blink, true);
             case 7:
                 S(reverse, true);
             case 9:
                 S(strike, true);
             case 21:
                 S(decoration, 2);
+            case 221:
+                S(bold, false);
+            case 222:
+                S(dim, false);
             case 22:
                 RANGE { cell->attrs.bold = false; cell->attrs.dim = false; } break;
             case 23:
                 S(italic, false);
             case 24:
                 S(decoration, 0);
+            case 25:
+                S(blink, false);
             case 27:
                 S(reverse, false);
             case 29:
@@ -203,6 +220,7 @@ END_ALLOW_CASE_RANGE
             case DECORATION_FG_CODE + 1:
                 SIMPLE(decoration_fg, 0);
         }
+        if (is_group) break;
     }
 #undef SET_COLOR
 #undef RANGE
@@ -213,10 +231,10 @@ END_ALLOW_CASE_RANGE
 const char*
 cursor_as_sgr(const Cursor *self) {
     GPUCell blank_cell = { 0 }, cursor_cell = {
-        .attrs = cursor_to_attrs(self, 1),
-        .fg = self->fg & COL_MASK,
-        .bg = self->bg & COL_MASK,
-        .decoration_fg = self->decoration_fg & COL_MASK,
+        .attrs = cursor_to_attrs(self),
+        .fg = self->sgr.fg & COL_MASK,
+        .bg = self->sgr.bg & COL_MASK,
+        .decoration_fg = self->sgr.decoration_fg & COL_MASK,
     };
     return cell_as_sgr(&cursor_cell, &blank_cell);
 }
@@ -237,7 +255,8 @@ void cursor_reset(Cursor *self) {
 void cursor_copy_to(Cursor *src, Cursor *dest) {
 #define CCY(x) dest->x = src->x;
     CCY(x); CCY(y); CCY(shape); CCY(non_blinking);
-    CCY(bold); CCY(italic); CCY(strikethrough); CCY(dim); CCY(reverse); CCY(decoration); CCY(fg); CCY(bg); CCY(decoration_fg);
+#undef CCY
+    memcpy(&dest->sgr, &src->sgr, sizeof(dest->sgr));
 }
 
 static PyObject*
@@ -246,25 +265,39 @@ copy(Cursor *self, PyObject*);
 
 // Boilerplate {{{
 
-BOOL_GETSET(Cursor, bold)
-BOOL_GETSET(Cursor, italic)
-BOOL_GETSET(Cursor, reverse)
-BOOL_GETSET(Cursor, strikethrough)
-BOOL_GETSET(Cursor, dim)
+#define SGR_BOOL_GETSET(x) \
+    static PyObject* x##_get(Cursor *self, void UNUSED *closure) { PyObject *ans = self->sgr.x ? Py_True : Py_False; Py_INCREF(ans); return ans; } \
+    static int x##_set(Cursor *self, PyObject *value, void UNUSED *closure) { if (value == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute"); return -1; } self->sgr.x = PyObject_IsTrue(value) ? true : false; return 0; }
+
+SGR_BOOL_GETSET(bold)
+SGR_BOOL_GETSET(italic)
+SGR_BOOL_GETSET(reverse)
+SGR_BOOL_GETSET(strikethrough)
+SGR_BOOL_GETSET(dim)
+#undef SGR_BOOL_GETSET
 
 static PyObject* blink_get(Cursor *self, void UNUSED *closure) { PyObject *ans = !self->non_blinking ? Py_True : Py_False; Py_INCREF(ans); return ans; }
 
 static int blink_set(Cursor *self, PyObject *value, void UNUSED *closure) { if (value == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute"); return -1; } self->non_blinking = PyObject_IsTrue(value) ? false : true; return 0; }
 
+static PyObject* text_blink_get(Cursor *self, void UNUSED *closure) { return Py_NewRef(self->sgr.blink ? Py_True : Py_False); }
+
+static int text_blink_set(Cursor *self, PyObject *value, void UNUSED *closure) { if (value == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute"); return -1; } self->sgr.blink = PyObject_IsTrue(value) ? false : true; return 0; }
+
+
+#define SGR_UINT_GETSET(x) \
+    static PyObject* x##_get(Cursor *self, void UNUSED *closure) { return PyLong_FromUnsignedLong((unsigned long)self->sgr.x); } \
+    static int x##_set(Cursor *self, PyObject *value, void UNUSED *closure) { if (value == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute"); return -1; } if (!PyLong_Check(value)) { PyErr_SetString(PyExc_TypeError, "value must be an int"); return -1; } self->sgr.x = PyLong_AsUnsignedLong(value); return 0; }
+
+SGR_UINT_GETSET(decoration)
+SGR_UINT_GETSET(fg)
+SGR_UINT_GETSET(bg)
+SGR_UINT_GETSET(decoration_fg)
 
 static PyMemberDef members[] = {
     {"x", T_UINT, offsetof(Cursor, x), 0, "x"},
     {"y", T_UINT, offsetof(Cursor, y), 0, "y"},
     {"shape", T_INT, offsetof(Cursor, shape), 0, "shape"},
-    {"decoration", T_UBYTE, offsetof(Cursor, decoration), 0, "decoration"},
-    {"fg", T_UINT, offsetof(Cursor, fg), 0, "fg"},
-    {"bg", T_UINT, offsetof(Cursor, bg), 0, "bg"},
-    {"decoration_fg", T_UINT, offsetof(Cursor, decoration_fg), 0, "decoration_fg"},
     {NULL}  /* Sentinel */
 };
 
@@ -275,6 +308,11 @@ static PyGetSetDef getseters[] = {
     GETSET(strikethrough)
     GETSET(dim)
     GETSET(blink)
+    GETSET(text_blink)
+    GETSET(decoration)
+    GETSET(fg)
+    GETSET(bg)
+    GETSET(decoration_fg)
     {NULL}  /* Sentinel */
 };
 
@@ -300,7 +338,7 @@ PyTypeObject Cursor_Type = {
     .tp_methods = methods,
     .tp_members = members,
     .tp_getset = getseters,
-    .tp_new = new,
+    .tp_new = new_cursor_object,
 };
 
 RICHCMP(Cursor)
@@ -321,8 +359,8 @@ copy(Cursor *self, PyObject *a UNUSED) {
     return (PyObject*)cursor_copy(self);
 }
 
-Cursor *alloc_cursor() {
-    return (Cursor*)new(&Cursor_Type, NULL, NULL);
+Cursor *alloc_cursor(void) {
+    return (Cursor*)new_cursor_object(&Cursor_Type, NULL, NULL);
 }
 
 INIT_TYPE(Cursor)
