@@ -836,7 +836,7 @@ PyInit_fast_data_types(void) {
     CellAttrs a;
 #define s(name, attr) { a.val = 0; a.attr = 1; PyModule_AddIntConstant(m, #name, shift_to_first_set_bit(a)); }
     s(BOLD, bold); s(ITALIC, italic); s(REVERSE, reverse); s(MARK, mark);
-    s(STRIKETHROUGH, strike); s(DIM, dim); s(DECORATION, decoration);
+    s(STRIKETHROUGH, strike); s(DIM, dim); s(DECORATION, decoration); s(BLINK, blink);
 #undef s
     PyModule_AddIntConstant(m, "MARK_MASK", MARK_MASK);
     PyModule_AddIntConstant(m, "DECORATION_MASK", DECORATION_MASK);
@@ -860,6 +860,10 @@ PyInit_fast_data_types(void) {
     PyModule_AddIntMacro(m, ESC_DCS);
     PyModule_AddIntMacro(m, ESC_PM);
     PyModule_AddIntMacro(m, TEXT_SIZE_CODE);
+    PyModule_AddIntMacro(m, COLOR_NOT_SET);
+    PyModule_AddIntMacro(m, COLOR_IS_SPECIAL);
+    PyModule_AddIntMacro(m, COLOR_IS_INDEX);
+    PyModule_AddIntMacro(m, COLOR_IS_RGB);
 #ifdef __APPLE__
     // Apple says its SHM_NAME_MAX but SHM_NAME_MAX is not actually declared in typical CrApple style.
     // This value is based on experimentation and from qsharedmemory.cpp in Qt
