@@ -29,9 +29,14 @@ typedef struct {
     GLint num_of_uniforms;
 } Program;
 
+typedef struct Viewport { unsigned left, top, width, height; } Viewport;
 
 void gl_init(void);
-void update_surface_size(int w, int h, GLuint offscreen_texture_id);
+const char* gl_version_string(void);
+void set_gpu_viewport(unsigned w, unsigned h);
+Viewport get_gpu_viewport(void);
+void draw_quad(bool blend, unsigned instance_count);
+void save_texture_as_png(uint32_t texture_id, const char *filename);
 void free_texture(GLuint *tex_id);
 void free_framebuffer(GLuint *fb_id);
 void remove_vao(ssize_t vao_idx);
@@ -56,3 +61,11 @@ void bind_vao_uniform_buffer(ssize_t vao_idx, size_t bufnum, GLuint block_index)
 void unbind_vertex_array(void);
 void unbind_program(void);
 GLuint compile_shaders(GLenum shader_type, GLsizei count, const GLchar * const * string);
+void save_viewport_using_top_left_origin(GLsizei x, GLsizei y, GLsizei width, GLsizei height, GLsizei full_framebuffer_height);
+void save_viewport_using_bottom_left_origin(GLsizei x, GLsizei y, GLsizei width, GLsizei height);
+void check_framebuffer_status_or_die(void);
+void restore_viewport(void);
+void bind_framebuffer_for_output(unsigned fbid);
+void set_framebuffer_to_use_for_output(unsigned fbid);
+void enable_scissor_using_top_left_origin(Viewport, unsigned);
+void disable_scissor(void);
