@@ -6,7 +6,7 @@ import os
 import shutil
 from collections.abc import Container, Iterable, Iterator, Sequence
 from contextlib import suppress
-from typing import Any, NamedTuple, TypedDict
+from typing import Any, Callable, NamedTuple, TypedDict
 
 from .boss import Boss
 from .child import Child
@@ -16,7 +16,7 @@ from .clipboard import set_clipboard_string, set_primary_selection
 from .fast_data_types import add_timer, get_boss, get_options, get_os_window_title, patch_color_profiles
 from .options.utils import env as parse_env
 from .tabs import Tab, TabManager
-from .types import OverlayType, run_once
+from .types import LayerShellConfig, OverlayType, run_once
 from .utils import get_editor, log_error, resolve_custom_file, which
 from .window import CwdRequest, CwdRequestType, Watchers, Window
 
@@ -26,6 +26,7 @@ class LaunchSpec(NamedTuple):
     args: list[str]
 
 
+# Options definition {{{
 env_docs = '''\
 type=list
 Environment variables to set in the child process. Can be specified multiple
@@ -81,7 +82,7 @@ of the active window in the tab is used as the tab title. The special value
 --type
 type=choices
 default=window
-choices=window,tab,os-window,overlay,overlay-main,background,clipboard,primary
+choices=window,tab,os-window,os-panel,overlay,overlay-main,background,clipboard,primary
 Where to launch the child process:
 
 :code:`window`
@@ -116,6 +117,11 @@ Where to launch the child process:
     These two are meant to work with :option:`--stdin-source <launch --stdin-source>` to copy
     data to the :italic:`system clipboard` or :italic:`primary selection`.
 
+:code:`os-panel`
+    Similar to :code:`os-window`, except that it creates the new OS Window as a desktop panel.
+    Only works on platforms that support this, such as Wayand compositors that support the layer
+    shell protocol. Use the :option:`kitten @ launch --os-panel` option to configure the panel.
+
 #placeholder_for_formatting#
 
 
@@ -136,6 +142,23 @@ oldest foreground process associated with the currently active window rather
 than the newest foreground process. Finally, the special value :code:`root`
 refers to the process that was originally started when the window was created.
 
+When opening in the same working directory as the current window causes the new
+window to connect to a remote host, you can use the :option:`--hold-after-ssh`
+flag to prevent the new window from closing when the connection is terminated.
+
+
+--add-to-session
+Add the newly created window/tab to the specified session. Use :code:`.` to add
+to the session of the :option:`source window <launch --source-window>`, if any. See :ref:`sessions`
+for what a session is and how to use one. By default, the window is added to the
+session of the :option:`source window <launch --source-window>` when :option:`launch --cwd`
+is set to use the the working directory from that window, otherwise the newly created window
+does not belong to any session. To force adding to no session, use the value :code:`!`.
+Adding a newly created window to a session is purely temporary, it does not change the actual
+session file, for that you have to resave the session. Note that using this flag in a launch
+command within a session file has no effect as the window is always added to the
+session belonging to that file.
+
 
 --env
 {env_docs}
@@ -151,6 +174,7 @@ times to set different user variables. Syntax: :code:`name=value`. Using
 --hold
 type=bool-set
 Keep the window open even after the command being executed exits, at a shell prompt.
+The shell will be run after the launched command exits.
 
 
 --copy-colors
@@ -209,7 +233,7 @@ of bias depends on the current layout.
 
 * Splits layout: The bias is interpreted as a percentage between 0 and 100.
 When splitting a window into two, the new window will take up the specified fraction
-of the space alloted to the original window and the original window will take up
+of the space allotted to the original window and the original window will take up
 the remainder of the space.
 
 * Vertical/horizontal layout: The bias is interpreted as adding/subtracting from the
@@ -377,7 +401,25 @@ the section on watchers in the launch command documentation: :ref:`watchers`.
 Relative paths are resolved relative to the :ref:`kitty config directory
 <confloc>`. Global watchers for all windows can be specified with
 :opt:`watcher` in :file:`kitty.conf`.
+
+
+--os-panel
+type=list
+Options to control the creation of desktop panels. Takes the same settings
+as the :doc:`panel kitten </kittens/panel>`, except for :option:`--override <kitty +kitten panel --override>`
+and :option:`--config <kitty +kitten panel --config>`. Can be specified multiple times.
+For example, to create a desktop panel at the bottom of the screen two lines high::
+
+    launch --type os-panel --os-panel lines=2 --os-panel edge=bottom sh -c "echo; echo; echo hello; sleep 5s"
+
+
+--hold-after-ssh
+type=bool-set
+When using :option:`--cwd`:code:`=current` or similar from a window that is running the ssh kitten,
+the new window will run a local shell after disconnecting from the remote host, when this option
+is specified.
 """
+# }}}
 
 
 def parse_launch_args(args: Sequence[str] | None = None) -> LaunchSpec:
@@ -401,19 +443,33 @@ def get_env(opts: LaunchCLIOptions, active_child: Child | None = None, base_env:
     return env
 
 
-def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Tab | None, next_to: Window | None) -> Tab:
+def layer_shell_config_from_panel_opts(panel_opts: Iterable[str]) -> LayerShellConfig:
+    from kittens.panel.main import layer_shell_config, parse_panel_args
+    args = [('' if x.startswith('--') else '--') + x for x in panel_opts]
+    try:
+        opts, _ = parse_panel_args(args)
+    except SystemExit as e:
+        raise ValueError(str(e))
+    return layer_shell_config(opts)
+
+
+def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Tab | None, next_to: Window | None, add_to_session: str) -> Tab:
 
     def create_tab(tm: TabManager | None = None) -> Tab:
         if tm is None:
-            oswid = boss.add_os_window(
-                wclass=opts.os_window_class,
-                wname=opts.os_window_name,
-                window_state=opts.os_window_state,
-                override_title=opts.os_window_title or None)
+            if opts.type == 'os-panel':
+                oswid = boss.add_os_panel(layer_shell_config_from_panel_opts(opts.os_panel), opts.os_window_class, opts.os_window_name)
+            else:
+                oswid = boss.add_os_window(
+                    wclass=opts.os_window_class,
+                    wname=opts.os_window_name,
+                    window_state=opts.os_window_state,
+                    override_title=opts.os_window_title or None)
             tm = boss.os_window_map[oswid]
         tab = tm.new_tab(empty_tab=True, location=opts.location)
         if opts.tab_title:
             tab.set_title(opts.tab_title)
+        tab.created_in_session_name = add_to_session
         return tab
 
     if opts.type == 'tab':
@@ -423,7 +479,7 @@ def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Tab | None, n
         if target_tab is not None:
             tm = target_tab.tab_manager_ref() or tm
         tab = create_tab(tm)
-    elif opts.type == 'os-window':
+    elif opts.type in ('os-window', 'os-panel'):
         tab = create_tab()
     else:
         if target_tab is not None:
@@ -489,6 +545,9 @@ def load_watch_modules(watchers: Iterable[str]) -> Watchers | None:
         w = m.get('on_color_scheme_preference_change')
         if callable(w):
             ans.on_color_scheme_preference_change.append(w)
+        w = m.get('on_tab_bar_dirty')
+        if callable(w):
+            ans.on_tab_bar_dirty.append(w)
     return ans
 
 
@@ -507,6 +566,7 @@ class LaunchKwds(TypedDict):
     stdin: bytes | None
     hold: bool
     bias: float | None
+    hold_after_ssh: bool
 
 
 def apply_colors(window: Window, spec: Sequence[str]) -> None:
@@ -565,6 +625,8 @@ def _launch(
     is_clone_launch: str = '',
     rc_from_window: Window | None = None,
     base_env: dict[str, str] | None = None,
+    child_death_callback: Callable[[int, Exception | None], None] | None = None,
+    startup_command_via_shell_integration: Sequence[str] | str = (),
 ) -> Window | None:
     source_window = boss.active_window_for_cwd
     if opts.source_window:
@@ -606,6 +668,7 @@ def _launch(
         'stdin': None,
         'hold': False,
         'bias': None,
+        'hold_after_ssh': False
     }
     spacing = {}
     if opts.spacing:
@@ -628,6 +691,11 @@ def _launch(
                 kw['cwd_from'] = CwdRequest(source_window, CwdRequestType.root)
         else:
             kw['cwd'] = opts.cwd
+    if opts.hold_after_ssh:
+        if opts.cwd not in ('current', 'last_reported', 'oldest'):
+            raise ValueError("--hold-after-ssh can only be supplied if --cwd=current or similar is also supplied")
+        kw['hold_after_ssh'] = True
+
     if opts.location != 'default':
         kw['location'] = opts.location
     if opts.copy_colors and source_window:
@@ -702,25 +770,50 @@ def _launch(
             raise ValueError('The cmd to run must be specified when running a background process')
         boss.run_background_process(
             cmd, cwd=kw['cwd'], cwd_from=kw['cwd_from'], env=env or None, stdin=kw['stdin'],
-            allow_remote_control=kw['allow_remote_control'], remote_control_passwords=kw['remote_control_passwords']
+            allow_remote_control=kw['allow_remote_control'], remote_control_passwords=kw['remote_control_passwords'],
+            notify_on_death=child_death_callback,
         )
     elif opts.type in ('clipboard', 'primary'):
         stdin = kw.get('stdin')
         if stdin is not None:
             if opts.type == 'clipboard':
                 set_clipboard_string(stdin)
+                boss.handle_clipboard_loss('clipboard')
             else:
                 set_primary_selection(stdin)
+                boss.handle_clipboard_loss('primary')
+        if child_death_callback is not None:
+            child_death_callback(0, None)
     else:
+        add_to_session = opts.add_to_session or ''
+        match add_to_session:
+            case '.':
+                add_to_session = source_window.created_in_session_name if source_window else ''
+            case '!':
+                add_to_session = ''
+            case '':
+                if kw['cwd_from'] is not None and source_window:
+                    add_to_session = source_window.created_in_session_name
         kw['hold'] = opts.hold
         if force_target_tab and target_tab is not None:
             tab = target_tab
         else:
-            tab = tab_for_window(boss, opts, target_tab, next_to)
+            tab = tab_for_window(boss, opts, target_tab, next_to, add_to_session)
         watchers = load_watch_modules(opts.watcher)
         with Window.set_ignore_focus_changes_for_new_windows(opts.keep_focus):
             new_window: Window = tab.new_window(
-                env=env or None, watchers=watchers or None, is_clone_launch=is_clone_launch, next_to=next_to, **kw)
+                env=env or None, watchers=watchers or None, is_clone_launch=is_clone_launch, next_to=next_to,
+                startup_command_via_shell_integration=startup_command_via_shell_integration, **kw)
+            new_window.created_in_session_name = add_to_session
+            if child_death_callback is not None:
+                boss.monitor_pid(new_window.child.pid or 0, child_death_callback)
+            if new_window.creation_spec:
+                if opts.watcher:
+                    new_window.creation_spec = new_window.creation_spec._replace(watchers=tuple(opts.watcher))
+                if opts.spacing:
+                    new_window.creation_spec = new_window.creation_spec._replace(spacing=tuple(opts.spacing))
+                if opts.color:
+                    new_window.creation_spec = new_window.creation_spec._replace(colors=tuple(opts.color))
         if spacing:
             patch_window_edges(new_window, spacing)
             tab.relayout()
@@ -736,7 +829,10 @@ def _launch(
         if opts.type == 'overlay-main':
             new_window.overlay_type = OverlayType.main
         if opts.var:
-            for key, val in parse_var(opts.var):
+            vars = tuple(parse_var(opts.var))
+            if new_window.creation_spec:
+                new_window.creation_spec = new_window.creation_spec._replace(user_vars=vars)
+            for key, val in vars:
                 new_window.set_user_var(key, val)
         return new_window
     return None
@@ -751,12 +847,16 @@ def launch(
     is_clone_launch: str = '',
     rc_from_window: Window | None = None,
     base_env: dict[str, str] | None = None,
+    child_death_callback: Callable[[int, Exception | None], None] | None = None,
+    startup_command_via_shell_integration: Sequence[str] | str = (),
 ) -> Window | None:
     active = boss.active_window
     if opts.keep_focus and active:
         orig, active.ignore_focus_changes = active.ignore_focus_changes, True
     try:
-        return _launch(boss, opts, args, target_tab, force_target_tab, is_clone_launch, rc_from_window, base_env)
+        return _launch(
+            boss, opts, args, target_tab, force_target_tab, is_clone_launch, rc_from_window, base_env,
+            child_death_callback, startup_command_via_shell_integration)
     finally:
         if opts.keep_focus and active:
             active.ignore_focus_changes = orig
@@ -766,7 +866,7 @@ def clone_safe_opts() -> frozenset[str]:
     return frozenset((
         'window_title', 'tab_title', 'type', 'keep_focus', 'cwd', 'env', 'var', 'hold',
         'location', 'os_window_class', 'os_window_name', 'os_window_title', 'os_window_state',
-        'logo', 'logo_position', 'logo_alpha', 'color', 'spacing', 'next_to',
+        'logo', 'logo_position', 'logo_alpha', 'color', 'spacing', 'next_to', 'hold_after_ssh'
     ))
 
 
@@ -913,6 +1013,42 @@ class EditCmd:
         window.write_to_child('KITTY_DATA_END\n')
 
 
+@run_once
+def excluded_env_vars() -> frozenset[str]:
+    return frozenset({
+        'HOME', 'LOGNAME', 'USER', 'PWD',
+        # some people export these. We want the shell rc files to recreate them
+        'PS0', 'PS1', 'PS2', 'PS3', 'PS4', 'RPS1', 'PROMPT_COMMAND', 'SHLVL',
+        # conda state env vars
+        'CONDA_SHLVL', 'CONDA_PREFIX', 'CONDA_PROMPT_MODIFIER', 'CONDA_EXE', 'CONDA_PYTHON_EXE', '_CE_CONDA', '_CE_M',
+        # skip SSH environment variables
+        'SSH_CLIENT', 'SSH_CONNECTION', 'SSH_ORIGINAL_COMMAND', 'SSH_TTY', 'SSH2_TTY',
+        'SSH_TUNNEL', 'SSH_USER_AUTH', 'SSH_AUTH_SOCK',
+        # Dont clone KITTY_WINDOW_ID
+        'KITTY_WINDOW_ID',
+        # Bash variables from "bind -x" and "complete -C" (needed not to confuse bash-preexec)
+        'READLINE_ARGUMENT', 'READLINE_LINE', 'READLINE_MARK', 'READLINE_POINT',
+        'COMP_LINE', 'COMP_POINT', 'COMP_TYPE',
+        # GPG gpg-agent
+        'GPG_TTY',
+        # Session variables of XDG
+        'XDG_SESSION_CLASS', 'XDG_SESSION_ID', 'XDG_SESSION_TYPE',
+        # Session variables of GNU Screen
+        'STY', 'WINDOW',
+        # Session variables of interactive shell plugins
+        'ATUIN_SESSION', 'ATUIN_HISTORY_ID',
+        'BLE_SESSION_ID', '_ble_util_fdlist_cloexec', '_ble_util_fdvars_export',
+        '_GITSTATUS_CLIENT_PID', '_GITSTATUS_REQ_FD', '_GITSTATUS_RESP_FD', 'GITSTATUS_DAEMON_PID',
+        'MCFLY_SESSION_ID',
+        'STARSHIP_SESSION_KEY',
+    })
+
+
+def is_excluded_env_var(x: str) -> bool:
+    # conda state env vars for multi-level virtual environments CONDA_PREFIX_*
+    return x in excluded_env_vars() or x.startswith('CONDA_PREFIX_')
+
+
 class CloneCmd:
 
     def __init__(self, msg: str) -> None:
@@ -943,36 +1079,7 @@ class CloneCmd:
                     env = parse_bash_env(v, self.bash_version)
                 else:
                     env = parse_null_env(v)
-                self.env = {k: v for k, v in env.items() if k not in {
-                    'HOME', 'LOGNAME', 'USER', 'PWD',
-                    # some people export these. We want the shell rc files to recreate them
-                    'PS0', 'PS1', 'PS2', 'PS3', 'PS4', 'RPS1', 'PROMPT_COMMAND', 'SHLVL',
-                    # conda state env vars
-                    'CONDA_SHLVL', 'CONDA_PREFIX', 'CONDA_PROMPT_MODIFIER', 'CONDA_EXE', 'CONDA_PYTHON_EXE', '_CE_CONDA', '_CE_M',
-                    # skip SSH environment variables
-                    'SSH_CLIENT', 'SSH_CONNECTION', 'SSH_ORIGINAL_COMMAND', 'SSH_TTY', 'SSH2_TTY',
-                    'SSH_TUNNEL', 'SSH_USER_AUTH', 'SSH_AUTH_SOCK',
-                    # Dont clone KITTY_WINDOW_ID
-                    'KITTY_WINDOW_ID',
-                    # Bash variables from "bind -x" and "complete -C" (needed not to confuse bash-preexec)
-                    'READLINE_ARGUMENT', 'READLINE_LINE', 'READLINE_MARK', 'READLINE_POINT',
-                    'COMP_LINE', 'COMP_POINT', 'COMP_TYPE',
-                    # GPG gpg-agent
-                    'GPG_TTY',
-                    # Session variables of XDG
-                    'XDG_SESSION_CLASS', 'XDG_SESSION_ID', 'XDG_SESSION_TYPE',
-                    # Session variables of GNU Screen
-                    'STY', 'WINDOW',
-                    # Session variables of interactive shell plugins
-                    'ATUIN_SESSION', 'ATUIN_HISTORY_ID',
-                    'BLE_SESSION_ID', '_ble_util_fdlist_cloexec', '_ble_util_fdvars_export',
-                    '_GITSTATUS_CLIENT_PID', '_GITSTATUS_REQ_FD', '_GITSTATUS_RESP_FD', 'GITSTATUS_DAEMON_PID',
-                    'MCFLY_SESSION_ID',
-                    'STARSHIP_SESSION_KEY',
-                } and not k.startswith((
-                    # conda state env vars for multi-level virtual environments
-                    'CONDA_PREFIX_',
-                ))}
+                self.env = {k: v for k, v in env.items() if not is_excluded_env_var(k)}
             elif k == 'cwd':
                 self.cwd = v
             elif k == 'history':
