@@ -1,15 +1,16 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
+from collections.abc import Generator, Iterable, Iterator, Sequence
 from functools import partial
 from itertools import repeat
-from typing import Any, Dict, Generator, Iterable, Iterator, List, NamedTuple, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, NamedTuple
 
 from kitty.borders import BorderColor
 from kitty.fast_data_types import Region, set_active_window, viewport_for_window
 from kitty.options.types import Options
-from kitty.types import Edges, WindowGeometry
-from kitty.typing import TypedDict, WindowType
+from kitty.types import Edges, WindowGeometry, WindowMapper
+from kitty.typing_compat import TypedDict, WindowType
 from kitty.window_list import WindowGroup, WindowList
 
 
@@ -20,10 +21,10 @@ class BorderLine(NamedTuple):
 
 class LayoutOpts:
 
-    def __init__(self, data: Dict[str, str]):
+    def __init__(self, data: dict[str, str]):
         pass
 
-    def serialized(self) -> Dict[str, Any]:
+    def serialized(self) -> dict[str, Any]:
         return {}
 
 
@@ -35,22 +36,23 @@ class LayoutData(NamedTuple):
     content_size: int = 0
 
 
-DecorationPairs = Sequence[Tuple[int, int]]
+DecorationPairs = Sequence[tuple[int, int]]
 LayoutDimension = Generator[LayoutData, None, None]
-ListOfWindows = List[WindowType]
+ListOfWindows = list[WindowType]
 
 
 class NeighborsMap(TypedDict):
-    left: List[int]
-    top: List[int]
-    right: List[int]
-    bottom: List[int]
+    left: list[int]
+    top: list[int]
+    right: list[int]
+    bottom: list[int]
 
 
 class LayoutGlobalData:
     draw_minimal_borders: bool = True
     draw_active_borders: bool = True
-    align_top_left: bool = False
+    alignment_x: int = 0
+    alignment_y: int = 0
 
     central: Region = Region((0, 0, 199, 199, 200, 200))
     cell_width: int = 20
@@ -60,7 +62,7 @@ class LayoutGlobalData:
 lgd = LayoutGlobalData()
 
 
-def idx_for_id(win_id: int, windows: Iterable[WindowType]) -> Optional[int]:
+def idx_for_id(win_id: int, windows: Iterable[WindowType]) -> int | None:
     for i, w in enumerate(windows):
         if w.id == win_id:
             return i
@@ -70,10 +72,11 @@ def idx_for_id(win_id: int, windows: Iterable[WindowType]) -> Optional[int]:
 def set_layout_options(opts: Options) -> None:
     lgd.draw_minimal_borders = opts.draw_minimal_borders and sum(opts.window_margin_width) == 0
     lgd.draw_active_borders = opts.active_border_color is not None
-    lgd.align_top_left = opts.placement_strategy == 'top-left'
+    lgd.alignment_x = -1 if opts.placement_strategy.endswith('left') else 1 if opts.placement_strategy.endswith('right') else 0
+    lgd.alignment_y = -1 if opts.placement_strategy.startswith('top') else 1 if opts.placement_strategy.startswith('bottom') else 0
 
 
-def convert_bias_map(bias: Dict[int, float], number_of_windows: int, number_of_cells: int) -> Sequence[float]:
+def convert_bias_map(bias: dict[int, float], number_of_windows: int, number_of_cells: int) -> Sequence[float]:
     cells_per_window, extra = divmod(number_of_cells, number_of_windows)
     cell_map = list(repeat(cells_per_window, number_of_windows))
     cell_map[-1] += extra
@@ -82,9 +85,9 @@ def convert_bias_map(bias: Dict[int, float], number_of_windows: int, number_of_c
 
 
 def calculate_cells_map(
-    bias: Union[None, Sequence[float], Dict[int, float]],
+    bias: None | Sequence[float] | dict[int, float],
     number_of_windows: int, number_of_cells: int
-) -> List[int]:
+) -> list[int]:
     if isinstance(bias, dict):
         bias = convert_bias_map(bias, number_of_windows, number_of_cells)
     cells_per_window = number_of_cells // number_of_windows
@@ -107,8 +110,8 @@ def calculate_cells_map(
 def layout_dimension(
     start_at: int, length: int, cell_length: int,
     decoration_pairs: DecorationPairs,
-    left_align: bool = False,
-    bias: Union[None, Sequence[float], Dict[int, float]] = None
+    alignment: int = 0,
+    bias: None | Sequence[float] | dict[int, float] = None
 ) -> LayoutDimension:
     number_of_windows = len(decoration_pairs)
     number_of_cells = length // cell_length
@@ -122,8 +125,10 @@ def layout_dimension(
     assert sum(cells_map) == number_of_cells
 
     extra = length - number_of_cells * cell_length - space_needed_for_decorations
-    pos = start_at
-    if not left_align:
+    pos = start_at  # start
+    if alignment > 0:  # end
+        pos += extra
+    elif alignment == 0:  # center
         pos += extra // 2
     last_i = len(cells_map) - 1
 
@@ -165,7 +170,7 @@ def blank_rects_for_window(wg: WindowGeometry) -> Generator[Rect, None, None]:
 
 def window_geometry(xstart: int, xnum: int, ystart: int, ynum: int, left: int, top: int, right: int, bottom: int) -> WindowGeometry:
     return WindowGeometry(
-        left=xstart, top=ystart, xnum=xnum, ynum=ynum,
+        left=xstart, top=ystart, xnum=max(0, xnum), ynum=max(0, ynum),
         right=xstart + lgd.cell_width * xnum, bottom=ystart + lgd.cell_height * ynum,
         spaces=Edges(left, top, right, bottom)
     )
@@ -175,24 +180,29 @@ def window_geometry_from_layouts(x: LayoutData, y: LayoutData) -> WindowGeometry
     return window_geometry(x.content_pos, x.cells_per_window, y.content_pos, y.cells_per_window, x.space_before, y.space_before, x.space_after, y.space_after)
 
 
-def layout_single_window(xdecoration_pairs: DecorationPairs, ydecoration_pairs: DecorationPairs, left_align: bool = False) -> WindowGeometry:
-    x = next(layout_dimension(lgd.central.left, lgd.central.width, lgd.cell_width, xdecoration_pairs, left_align=lgd.align_top_left))
-    y = next(layout_dimension(lgd.central.top, lgd.central.height, lgd.cell_height, ydecoration_pairs, left_align=lgd.align_top_left))
+def layout_single_window(
+    xdecoration_pairs: DecorationPairs,
+    ydecoration_pairs: DecorationPairs,
+    xalignment: int = 0,
+    yalignment: int = 0,
+) -> WindowGeometry:
+    x = next(layout_dimension(lgd.central.left, lgd.central.width, lgd.cell_width, xdecoration_pairs, alignment=xalignment))
+    y = next(layout_dimension(lgd.central.top, lgd.central.height, lgd.cell_height, ydecoration_pairs, alignment=yalignment))
     return window_geometry_from_layouts(x, y)
 
 
-def safe_increment_bias(old_val: float, increment: float) -> float:
+def safe_increment_bias(old_val: float, increment: float = 0) -> float:
     return max(0.1, min(old_val + increment, 0.9))
 
 
-def normalize_biases(biases: List[float]) -> List[float]:
+def normalize_biases(biases: list[float]) -> list[float]:
     s = sum(biases)
     if s == 1.0:
         return biases
     return [x/s for x in biases]
 
 
-def distribute_indexed_bias(base_bias: Sequence[float], index_bias_map: Dict[int, float]) -> Sequence[float]:
+def distribute_indexed_bias(base_bias: Sequence[float], index_bias_map: dict[int, float]) -> Sequence[float]:
     if not index_bias_map:
         return base_bias
     ans = list(base_bias)
@@ -205,6 +215,14 @@ def distribute_indexed_bias(base_bias: Sequence[float], index_bias_map: Dict[int
     return normalize_biases(ans)
 
 
+def create_window_id_map_for_unserialize(all_windows: WindowList) -> dict[int, int]:
+    window_id_map = {}
+    for w in all_windows:
+        if w.serialized_id:
+            window_id_map[w.serialized_id] = w.id
+    return window_id_map
+
+
 class Layout:
 
     name: str = ''
@@ -214,22 +232,26 @@ class Layout:
     only_active_window_visible = False
 
     def __init__(self, os_window_id: int, tab_id: int, layout_opts: str = '') -> None:
-        self.os_window_id = os_window_id
-        self.tab_id = tab_id
-        self.set_active_window_in_os_window = partial(set_active_window, os_window_id, tab_id)
+        self.set_owner(os_window_id, tab_id)
         # A set of rectangles corresponding to the blank spaces at the edges of
         # this layout, i.e. spaces that are not covered by any window
-        self.blank_rects: List[Rect] = []
+        self.blank_rects: list[Rect] = []
         self.layout_opts = self.parse_layout_opts(layout_opts)
         assert self.name is not None
         self.full_name = f'{self.name}:{layout_opts}' if layout_opts else self.name
         self.remove_all_biases()
 
-    def bias_increment_for_cell(self, all_windows: WindowList, window_id: int, is_horizontal: bool) -> float:
-        self._set_dimensions()
-        return self.calculate_bias_increment_for_a_single_cell(all_windows, window_id, is_horizontal)
+    def set_owner(self, os_window_id: int, tab_id: int) -> None:
+        # Useful when moving a layout from one tab to another typically a detached tab being re-attached
+        self.os_window_id = os_window_id
+        self.tab_id = tab_id
+        self.set_active_window_in_os_window = partial(set_active_window, os_window_id, tab_id)
 
-    def calculate_bias_increment_for_a_single_cell(self, all_windows: WindowList, window_id: int, is_horizontal: bool) -> float:
+    def bias_increment_for_cell(self, all_windows: WindowList, is_horizontal: bool) -> float:
+        self._set_dimensions()
+        return self.calculate_bias_increment_for_a_single_cell(all_windows, is_horizontal)
+
+    def calculate_bias_increment_for_a_single_cell(self, all_windows: WindowList, is_horizontal: bool) -> float:
         if is_horizontal:
             return (lgd.cell_width + 1) / lgd.central.width
         return (lgd.cell_height + 1) / lgd.central.height
@@ -242,12 +264,12 @@ class Layout:
 
     def modify_size_of_window(self, all_windows: WindowList, window_id: int, increment: float, is_horizontal: bool = True) -> bool:
         idx = all_windows.group_idx_for_window(window_id)
-        if idx is None:
+        if idx is None or not increment:
             return False
         return self.apply_bias(idx, increment, all_windows, is_horizontal)
 
-    def parse_layout_opts(self, layout_opts: Optional[str] = None) -> LayoutOpts:
-        data: Dict[str, str] = {}
+    def parse_layout_opts(self, layout_opts: str | None = None) -> LayoutOpts:
+        data: dict[str, str] = {}
         if layout_opts:
             for x in layout_opts.split(';'):
                 k, v = x.partition('=')[::2]
@@ -255,7 +277,7 @@ class Layout:
                     data[k] = v
         return type(self.layout_opts)(data)
 
-    def nth_window(self, all_windows: WindowList, num: int) -> Optional[WindowType]:
+    def nth_window(self, all_windows: WindowList, num: int) -> WindowType | None:
         return all_windows.active_window_in_nth_group(num, clamp=True)
 
     def activate_nth_window(self, all_windows: WindowList, num: int) -> None:
@@ -279,20 +301,26 @@ class Layout:
         return all_windows.move_window_group(to_group=group)
 
     def add_window(
-        self, all_windows: WindowList, window: WindowType, location: Optional[str] = None,
-        overlay_for: Optional[int] = None, put_overlay_behind: bool = False
-    ) -> None:
-        if overlay_for is not None and overlay_for in all_windows:
-            all_windows.add_window(window, group_of=overlay_for, head_of_group=put_overlay_behind)
-            return
+        self, all_windows: WindowList, window: WindowType, location: str | None = None,
+        overlay_for: int | None = None, put_overlay_behind: bool = False, bias: float | None = None,
+        next_to: WindowType | None = None,
+    ) -> WindowType | None:
+        if overlay_for is not None:
+            underlay = all_windows.id_map.get(overlay_for)
+            if underlay is not None:
+                window.margin, window.padding = underlay.margin.copy(), underlay.padding.copy()
+                all_windows.add_window(window, group_of=overlay_for, head_of_group=put_overlay_behind)
+                return underlay
         if location == 'neighbor':
             location = 'after'
-        self.add_non_overlay_window(all_windows, window, location)
+        self.add_non_overlay_window(all_windows, window, location, bias, next_to)
+        return None
 
-    def add_non_overlay_window(self, all_windows: WindowList, window: WindowType, location: Optional[str]) -> None:
-        next_to: Optional[WindowType] = None
+    def add_non_overlay_window(
+        self, all_windows: WindowList, window: WindowType, location: str | None, bias: float | None = None, next_to: WindowType | None = None
+    ) -> None:
         before = False
-        next_to = all_windows.active_window
+        next_to = next_to or all_windows.active_window
         if location is not None:
             if location in ('after', 'vsplit', 'hsplit'):
                 pass
@@ -304,6 +332,21 @@ class Layout:
             elif location == 'last':
                 next_to = None
         all_windows.add_window(window, next_to=next_to, before=before)
+        if bias is not None:
+            idx = all_windows.group_idx_for_window(window)
+            if idx is not None:
+                self._set_dimensions()
+                self._bias_slot(all_windows, idx, bias)
+
+    def _bias_slot(self, all_windows: WindowList, idx: int, bias: float) -> bool:
+        fractional_bias = max(10, min(abs(bias), 90)) / 100
+        h, v = self.calculate_bias_increment_for_a_single_cell(all_windows, True), self.calculate_bias_increment_for_a_single_cell(all_windows, False)
+        nh, nv = lgd.central.width / lgd.cell_width, lgd.central.height / lgd.cell_height
+        f = max(-90, min(bias, 90)) / 100.
+        return self.bias_slot(all_windows, idx, fractional_bias, h * nh *f, v * nv * f)
+
+    def bias_slot(self, all_windows: WindowList, idx: int, fractional_bias: float, cell_increment_bias_h: float, cell_increment_bias_v: float) -> bool:
+        return False
 
     def update_visibility(self, all_windows: WindowList) -> None:
         active_window = all_windows.active_window
@@ -330,17 +373,17 @@ class Layout:
             wg.decoration('top', border_mult=bw, is_single_window=True),
             wg.decoration('bottom', border_mult=bw, is_single_window=True),
         ),)
-        geom = layout_single_window(xdecoration_pairs, ydecoration_pairs, left_align=lgd.align_top_left)
+        geom = layout_single_window(xdecoration_pairs, ydecoration_pairs, xalignment=lgd.alignment_x, yalignment=lgd.alignment_y)
         wg.set_geometry(geom)
-        if add_blank_rects and wg:
+        if add_blank_rects:
             self.blank_rects.extend(blank_rects_for_window(geom))
 
     def xlayout(
         self,
         groups: Iterator[WindowGroup],
-        bias: Union[None, Sequence[float], Dict[int, float]] = None,
-        start: Optional[int] = None,
-        size: Optional[int] = None,
+        bias: None | Sequence[float] | dict[int, float] = None,
+        start: int | None = None,
+        size: int | None = None,
         offset: int = 0,
         border_mult: int = 1
     ) -> LayoutDimension:
@@ -352,14 +395,14 @@ class Layout:
             start = lgd.central.left
         if size is None:
             size = lgd.central.width
-        return layout_dimension(start, size, lgd.cell_width, decoration_pairs, bias=bias, left_align=lgd.align_top_left)
+        return layout_dimension(start, size, lgd.cell_width, decoration_pairs, bias=bias, alignment=lgd.alignment_x)
 
     def ylayout(
         self,
         groups: Iterator[WindowGroup],
-        bias: Union[None, Sequence[float], Dict[int, float]] = None,
-        start: Optional[int] = None,
-        size: Optional[int] = None,
+        bias: None | Sequence[float] | dict[int, float] = None,
+        start: int | None = None,
+        size: int | None = None,
         offset: int = 0,
         border_mult: int = 1
     ) -> LayoutDimension:
@@ -371,7 +414,7 @@ class Layout:
             start = lgd.central.top
         if size is None:
             size = lgd.central.height
-        return layout_dimension(start, size, lgd.cell_height, decoration_pairs, bias=bias, left_align=lgd.align_top_left)
+        return layout_dimension(start, size, lgd.cell_height, decoration_pairs, bias=bias, alignment=lgd.alignment_y)
 
     def set_window_group_geometry(self, wg: WindowGroup, xl: LayoutData, yl: LayoutData) -> WindowGeometry:
         geom = window_geometry_from_layouts(xl, yl)
@@ -385,7 +428,7 @@ class Layout:
     def neighbors_for_window(self, window: WindowType, windows: WindowList) -> NeighborsMap:
         return {'left': [], 'right': [], 'top': [], 'bottom': []}
 
-    def compute_needs_borders_map(self, all_windows: WindowList) -> Dict[int, bool]:
+    def compute_needs_borders_map(self, all_windows: WindowList) -> dict[int, bool]:
         return all_windows.compute_needs_borders_map(lgd.draw_active_borders)
 
     def get_minimal_borders(self, windows: WindowList) -> Generator[BorderLine, None, None]:
@@ -396,8 +439,30 @@ class Layout:
         return
         yield BorderLine()  # type: ignore
 
-    def layout_action(self, action_name: str, args: Sequence[str], all_windows: WindowList) -> Optional[bool]:
+    def layout_action(self, action_name: str, args: Sequence[str], all_windows: WindowList) -> bool | None:
         pass
 
-    def layout_state(self) -> Dict[str, Any]:
+    def layout_state(self) -> dict[str, Any]:
         return {}
+
+    def set_layout_state(self, layout_state: dict[str, Any], map_group_id: WindowMapper) -> bool:
+        return True
+
+    def serialize(self, all_windows: WindowList) -> dict[str, Any]:
+        ans = self.layout_state()
+        ans['opts'] = self.layout_opts.serialized()
+        ans['class'] = self.__class__.__name__
+        ans['all_windows'] = all_windows.serialize_layout_state()
+        return ans
+
+    def unserialize(
+        self, s: dict[str, Any], all_windows: WindowList,
+        window_id_mapper: Callable[[WindowList], dict[int, int]] = create_window_id_map_for_unserialize,
+    ) -> bool:
+        if s.get('class') != self.__class__.__name__:
+            return False
+        window_id_map = create_window_id_map_for_unserialize(all_windows)
+        m = all_windows.unserialize_layout_state(s['all_windows'], window_id_map)
+        if m is None:
+            return False
+        return self.set_layout_state(s, m.get)
