@@ -1,32 +1,37 @@
 #!/usr/bin/env python
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
+import json
 import locale
 import os
 import shutil
 import sys
+from collections.abc import Generator, Sequence
 from contextlib import contextmanager, suppress
-from typing import Dict, Generator, List, Optional, Sequence, Tuple
+from gettext import gettext as _
 
 from .borders import load_borders_program
 from .boss import Boss
 from .child import set_default_env, set_LANG_in_default_env
 from .cli import create_opts, parse_args
 from .cli_stub import CLIOptions
+from .colors import theme_colors
 from .conf.utils import BadLine
 from .config import cached_values_for
 from .constants import (
     appname,
     beam_cursor_data_file,
     clear_handled_signals,
-    config_dir,
     glfw_path,
     is_macos,
+    is_quick_access_terminal_app,
     is_wayland,
     kitten_exe,
     kitty_exe,
+    launched_by_launch_services,
     logo_png_file,
     running_in_kitty,
+    supports_window_occlusion,
     website_url,
 )
 from .fast_data_types import (
@@ -35,34 +40,35 @@ from .fast_data_types import (
     SingleKey,
     create_os_window,
     free_font_data,
+    glfw_get_monitor_names,
+    glfw_get_monitor_workarea,
     glfw_init,
     glfw_terminate,
+    grab_keyboard,
+    is_layer_shell_supported,
     load_png_data,
     mask_kitty_signals_process_wide,
+    run_at_exit_cleanup_functions,
     set_custom_cursor,
     set_default_window_icon,
     set_options,
+    set_use_os_log,
 )
-from .fonts.box_drawing import set_scale
-from .fonts.render import set_font_family
+from .fonts.render import dump_font_debug, set_font_family
 from .options.types import Options
 from .options.utils import DELETE_ENV_VAR
 from .os_window_size import edge_spacing, initial_window_size_func
 from .session import create_sessions, get_os_window_sizing_data
 from .shaders import CompileError, load_shader_programs
-from .types import LayerShellConfig, SingleInstanceData
+from .types import LayerShellConfig
 from .utils import (
     cleanup_ssh_control_masters,
-    detach,
     expandvars,
     get_custom_window_icon,
     log_error,
     parse_os_window_state,
     safe_mtime,
-    shlex_split,
-    single_instance,
     startup_notification_handler,
-    unix_socket_paths,
 )
 
 
@@ -81,62 +87,19 @@ def set_custom_ibeam_cursor() -> None:
         log_error(f'Failed to set custom beam cursor with error: {e}')
 
 
-def talk_to_instance(args: CLIOptions) -> None:
-    import json
-    import socket
-    session_data = ''
-    if args.session == '-':
-        session_data = sys.stdin.read()
-    elif args.session == 'none':
-        session_data = 'none'
-    elif args.session:
-        with open(args.session) as f:
-            session_data = f.read()
-
-    data: SingleInstanceData = {
-        'cmd': 'new_instance', 'args': tuple(sys.argv), 'cmdline_args_for_open': getattr(sys, 'cmdline_args_for_open', ()),
-        'cwd': os.getcwd(), 'session_data': session_data, 'environ': dict(os.environ), 'notify_on_os_window_death': None
-    }
-    notify_socket = None
-    if args.wait_for_single_instance_window_close:
-        address = f'\0{appname}-os-window-close-notify-{os.getpid()}-{os.geteuid()}'
-        notify_socket = socket.socket(family=socket.AF_UNIX)
-        try:
-            notify_socket.bind(address)
-        except FileNotFoundError:
-            for address in unix_socket_paths(address[1:], ext='.sock'):
-                notify_socket.bind(address)
-                break
-        data['notify_on_os_window_death'] = address
-        notify_socket.listen()
-
-    sdata = json.dumps(data, ensure_ascii=False).encode('utf-8')
-    assert single_instance.socket is not None
-    single_instance.socket.sendall(sdata)
-    with suppress(OSError):
-        single_instance.socket.shutdown(socket.SHUT_RDWR)
-    single_instance.socket.close()
-
-    if args.wait_for_single_instance_window_close:
-        assert notify_socket is not None
-        conn = notify_socket.accept()[0]
-        conn.recv(1)
-        with suppress(OSError):
-            conn.shutdown(socket.SHUT_RDWR)
-        conn.close()
-
-
-def load_all_shaders(semi_transparent: bool = False) -> None:
+def load_all_shaders() -> None:
     try:
-        load_shader_programs(semi_transparent)
+        load_shader_programs()
         load_borders_program()
     except CompileError as err:
         raise SystemExit(err)
 
 
-def init_glfw_module(glfw_module: str, debug_keyboard: bool = False, debug_rendering: bool = False, wayland_enable_ime: bool = True) -> None:
-    if not glfw_init(glfw_path(glfw_module), edge_spacing, debug_keyboard, debug_rendering, wayland_enable_ime):
+def init_glfw_module(glfw_module: str = 'wayland', debug_keyboard: bool = False, debug_rendering: bool = False, wayland_enable_ime: bool = True) -> None:
+    ok, swo = glfw_init(glfw_path(glfw_module), edge_spacing, debug_keyboard, debug_rendering, wayland_enable_ime)
+    if not ok:
         raise SystemExit('GLFW initialization failed')
+    supports_window_occlusion(swo)
 
 
 def init_glfw(opts: Options, debug_keyboard: bool = False, debug_rendering: bool = False) -> str:
@@ -146,8 +109,8 @@ def init_glfw(opts: Options, debug_keyboard: bool = False, debug_rendering: bool
 
 
 def get_macos_shortcut_for(
-    func_map: Dict[Tuple[str, ...], List[SingleKey]], defn: str = 'new_os_window', lookup_name: str = ''
-) -> Optional[SingleKey]:
+    func_map: dict[tuple[str, ...], list[SingleKey]], defn: str = 'new_os_window', lookup_name: str = ''
+) -> SingleKey | None:
     # for maximum robustness we should use opts.alias_map to resolve
     # aliases however this requires parsing everything on startup which could
     # be potentially slow. Lets just hope the user doesn't alias these
@@ -199,23 +162,25 @@ def get_icon128_path(base_path: str) -> str:
     return f'{path}-128{ext}'
 
 
-def set_x11_window_icon() -> None:
+def set_window_icon() -> None:
     custom_icon_path = get_custom_window_icon()[1]
+    is_x11 = not is_macos and not is_wayland()
     try:
         if custom_icon_path is not None:
             custom_icon128_path = get_icon128_path(custom_icon_path)
-            if safe_mtime(custom_icon128_path) is None:
-                set_default_window_icon(custom_icon_path)
-            else:
+            if is_x11 and safe_mtime(custom_icon128_path) is not None:
                 set_default_window_icon(custom_icon128_path)
+            else:
+                set_default_window_icon(custom_icon_path)
         else:
-            set_default_window_icon(get_icon128_path(logo_png_file))
+            if is_x11:
+                set_default_window_icon(get_icon128_path(logo_png_file))
     except ValueError as err:
         log_error(err)
 
 
-def set_cocoa_global_shortcuts(opts: Options) -> Dict[str, SingleKey]:
-    global_shortcuts: Dict[str, SingleKey] = {}
+def set_cocoa_global_shortcuts(opts: Options) -> dict[str, SingleKey]:
+    global_shortcuts: dict[str, SingleKey] = {}
     if is_macos:
         from collections import defaultdict
         func_map = defaultdict(list)
@@ -237,6 +202,15 @@ def set_cocoa_global_shortcuts(opts: Options) -> Dict[str, SingleKey]:
         val = get_macos_shortcut_for(func_map, 'clear_terminal to_cursor active', lookup_name='clear_terminal_and_scrollback')
         if val is not None:
             global_shortcuts['clear_terminal_and_scrollback'] = val
+        val = get_macos_shortcut_for(func_map, 'clear_terminal scrollback active', lookup_name='clear_scrollback')
+        if val is not None:
+            global_shortcuts['clear_scrollback'] = val
+        val = get_macos_shortcut_for(func_map, 'clear_terminal to_cursor_scroll active', lookup_name='clear_screen')
+        if val is not None:
+            global_shortcuts['clear_screen'] = val
+        val = get_macos_shortcut_for(func_map, 'clear_terminal last_command active', lookup_name='clear_last_command')
+        if val is not None:
+            global_shortcuts['clear_last_command'] = val
         val = get_macos_shortcut_for(func_map, 'load_config_file', lookup_name='reload_config')
         if val is not None:
             global_shortcuts['reload_config'] = val
@@ -246,7 +220,47 @@ def set_cocoa_global_shortcuts(opts: Options) -> Dict[str, SingleKey]:
     return global_shortcuts
 
 
-def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ()) -> None:
+_is_panel_kitten = False
+
+
+def is_panel_kitten() -> bool:
+    return _is_panel_kitten
+
+
+def list_monitors(json_output: bool = False) -> None:
+    monitor_names = glfw_get_monitor_names()
+    has_descriptions = False
+    for (name, desc) in monitor_names:
+        if desc:
+            has_descriptions = True
+            break
+
+    if json_output:
+        if has_descriptions:
+            monitors_list_of_dict = [{'name': name, 'description': desc} for name, desc in monitor_names]
+        else:
+            monitors_list_of_dict = [{'name': name} for name, _ in monitor_names]
+        json.dump(monitors_list_of_dict, sys.stdout, indent=2, sort_keys=True)
+        print()
+        return
+
+    isatty = sys.stdout.isatty()
+    for (name, desc) in monitor_names:
+        if isatty:
+            name = f'\x1b[32m{name}\x1b[39m'  # ]]
+        print(name)
+        if desc:
+            print(f'\t{desc}')
+        if has_descriptions:
+            print()
+
+
+def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = (), talk_fd: int = -1) -> None:
+    global _is_panel_kitten
+    _is_panel_kitten = run_app.cached_values_name == 'panel'
+    if _is_panel_kitten and run_app.layer_shell_config and run_app.layer_shell_config.output_name in ('list', 'listjson'):
+        list_monitors(run_app.layer_shell_config.output_name == 'listjson')
+        return
     if is_macos:
         global_shortcuts = set_cocoa_global_shortcuts(opts)
         if opts.macos_custom_beam_cursor:
@@ -254,27 +268,51 @@ def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ())
         set_macos_app_custom_icon()
     else:
         global_shortcuts = {}
-        if not is_wayland():  # no window icons on wayland
-            set_x11_window_icon()
-
+        set_window_icon()
+    if _is_panel_kitten and not is_layer_shell_supported():
+        raise SystemExit('Cannot create panels as the window manager/compositor does not support the necessary protocols')
+    pos_x, pos_y = None, None
+    if args.grab_keyboard:
+        grab_keyboard(True)
     with cached_values_for(run_app.cached_values_name) as cached_values:
-        startup_sessions = tuple(create_sessions(opts, args, default_session=opts.startup_session))
+        if not _is_panel_kitten and not is_wayland():
+            if opts.remember_window_position:
+                cached_workarea = tuple(tuple(x) for x in cached_values.get('monitor-workarea', ()))
+                if cached_workarea and glfw_get_monitor_workarea() == tuple(cached_workarea):
+                    pos_x, pos_y = cached_values.get('window-pos', (None, None))
+            if args.position:
+                pos_x, pos_y = map(int, args.position.lower().partition('x')[::2])
+        startup_session_error: tuple[Exception, str] | None = None
+        try:
+            startup_sessions = tuple(create_sessions(opts, args, default_session=opts.startup_session))
+        except Exception as e:
+            startup_session_error = (e, (getattr(args, 'session', '') or opts.startup_session or ''))
+            if getattr(args, 'session', ''):
+                args.session = ''
+            startup_sessions = tuple(create_sessions(opts, args))
         wincls = (startup_sessions[0].os_window_class if startup_sessions else '') or args.cls or appname
+        winname = (startup_sessions[0].os_window_name if startup_sessions else '') or args.name or wincls or appname
         window_state = (args.start_as if args.start_as and args.start_as != 'normal' else None) or (
             getattr(startup_sessions[0], 'os_window_state', None) if startup_sessions else None
         )
         wstate = parse_os_window_state(window_state) if window_state is not None else None
+
         with startup_notification_handler(extra_callback=run_app.first_window_callback) as pre_show_callback:
             window_id = create_os_window(
                     run_app.initial_window_size_func(get_os_window_sizing_data(opts, startup_sessions[0] if startup_sessions else None), cached_values),
                     pre_show_callback,
-                    args.title or appname, args.name or args.cls or appname,
-                    wincls, wstate, load_all_shaders, disallow_override_title=bool(args.title), layer_shell_config=run_app.layer_shell_config)
-        boss = Boss(opts, args, cached_values, global_shortcuts)
+                    args.title or appname, winname,
+                    wincls, wstate, load_all_shaders, disallow_override_title=bool(args.title), layer_shell_config=run_app.layer_shell_config, x=pos_x, y=pos_y)
+        boss = Boss(opts, args, cached_values, global_shortcuts, talk_fd)
         boss.start(window_id, startup_sessions)
+        if args.debug_font_fallback:
+            dump_font_debug()
         if bad_lines or boss.misc_config_errors:
             boss.show_bad_config_lines(bad_lines, boss.misc_config_errors)
             boss.misc_config_errors = []
+        if startup_session_error:
+            boss.show_error(_('The startup session was invalid'), _(
+                'Loading the start session file {0} failed, with error:\n{1}').format(startup_session_error[1], startup_session_error[0]))
         try:
             boss.child_monitor.main_loop()
         finally:
@@ -286,15 +324,16 @@ class AppRunner:
     def __init__(self) -> None:
         self.cached_values_name = 'main'
         self.first_window_callback = lambda window_handle: None
-        self.layer_shell_config: Optional[LayerShellConfig] = None
+        self.layer_shell_config: LayerShellConfig | None = None
         self.initial_window_size_func = initial_window_size_func
 
-    def __call__(self, opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ()) -> None:
-        set_scale(opts.box_drawing_scale)
+    def __call__(self, opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = (), talk_fd: int = -1) -> None:
+        if theme_colors.refresh():
+            theme_colors.patch_opts(opts, args.debug_rendering)
         set_options(opts, is_wayland(), args.debug_rendering, args.debug_font_fallback)
         try:
-            set_font_family(opts, debug_font_matching=args.debug_font_fallback)
-            _run_app(opts, args, bad_lines)
+            set_font_family(opts, add_builtin_nerd_font=True)
+            _run_app(opts, args, bad_lines, talk_fd)
         finally:
             set_options(None)
             free_font_data()  # must free font data before glfw/freetype/fontconfig/opengl etc are finalized
@@ -354,19 +393,6 @@ def setup_profiling() -> Generator[None, None, None]:
             print('To view the graphical call data, use: kcachegrind', cg)
 
 
-def macos_cmdline(argv_args: List[str]) -> List[str]:
-    try:
-        with open(os.path.join(config_dir, 'macos-launch-services-cmdline')) as f:
-            raw = f.read()
-    except FileNotFoundError:
-        return argv_args
-    raw = raw.strip()
-    ans = list(shlex_split(raw))
-    if ans and ans[0] == 'kitty':
-        del ans[0]
-    return ans + argv_args
-
-
 def expand_listen_on(listen_on: str, from_config_file: bool) -> str:
     if from_config_file and listen_on == 'none':
         return ''
@@ -429,7 +455,7 @@ def ensure_kitten_in_path() -> None:
     os.environ['PATH'] = prepend_if_not_present(os.path.dirname(correct_kitten), env_path)
 
 
-def setup_manpath(env: Dict[str, str]) -> None:
+def setup_manpath(env: dict[str, str]) -> None:
     # Ensure kitty manpages are available in frozen builds
     if not getattr(sys, 'frozen', False):
         return
@@ -483,51 +509,52 @@ def set_locale() -> None:
             set_LANG_in_default_env(old_lang)
 
 
-def _main() -> None:
+def kitty_main(called_from_panel: bool = False) -> None:
     running_in_kitty(True)
 
     args = sys.argv[1:]
-    if is_macos and os.environ.pop('KITTY_LAUNCHED_BY_LAUNCH_SERVICES', None) == '1':
-        os.chdir(os.path.expanduser('~'))
-        args = macos_cmdline(args)
-        getattr(sys, 'kitty_run_data')['launched_by_launch_services'] = True
     try:
         cwd_ok = os.path.isdir(os.getcwd())
     except Exception:
         cwd_ok = False
     if not cwd_ok:
         os.chdir(os.path.expanduser('~'))
+    cli_flags = None
     if getattr(sys, 'cmdline_args_for_open', False):
-        usage: Optional[str] = 'file_or_url ...'
-        appname: Optional[str] = 'kitty +open'
-        msg: Optional[str] = (
+        usage: str | None = 'file_or_url ...'
+        appname: str | None = 'kitty +open'
+        msg: str | None = (
             'Run kitty and open the specified files or URLs in it, using launch-actions.conf. For details'
             ' see https://sw.kovidgoyal.net/kitty/open_actions/#scripting-the-opening-of-files-with-kitty-on-macos'
             '\n\nAll the normal kitty options can be used.')
     else:
+        if not called_from_panel:
+            cli_flags = getattr(sys, 'kitty_run_data', {}).get('cli_flags', None)
         usage = msg = appname = None
-    cli_opts, rest = parse_args(args=args, result_class=CLIOptions, usage=usage, message=msg, appname=appname)
+    cli_opts, rest = parse_args(args=args, result_class=CLIOptions, usage=usage, message=msg, appname=appname, preparsed_from_c=cli_flags)
     if getattr(sys, 'cmdline_args_for_open', False):
         setattr(sys, 'cmdline_args_for_open', rest)
         cli_opts.args = []
     else:
         cli_opts.args = rest
+    talk_fd = -1
+    if cli_opts.single_instance:
+        si_data = os.environ.pop('KITTY_SI_DATA', '')
+        if si_data:
+            talk_fd = int(si_data)
+
     if cli_opts.detach:
         if cli_opts.session == '-':
             from .session import PreReadSession
-            cli_opts.session = PreReadSession(sys.stdin.read(), os.environ)
-        detach()
+            cli_opts.session = PreReadSession(sys.stdin.read(), os.environ, '-', os.path.join(os.getcwd(), '-'))
     if cli_opts.replay_commands:
         from kitty.client import main as client_main
         client_main(cli_opts.replay_commands)
         return
-    if cli_opts.single_instance:
-        is_first = single_instance(cli_opts.instance_group)
-        if not is_first:
-            talk_to_instance(cli_opts)
-            return
-    bad_lines: List[BadLine] = []
+    bad_lines: list[BadLine] = []
     opts = create_opts(cli_opts, accumulate_bad_lines=bad_lines)
+    if is_quick_access_terminal_app:
+        opts.macos_hide_from_tasks = True
     setup_environment(opts, cli_opts)
 
     # set_locale on macOS uses cocoa APIs when LANG is not set, so we have to
@@ -551,17 +578,31 @@ def _main() -> None:
     try:
         with setup_profiling():
             # Avoid needing to launch threads to reap zombies
-            run_app(opts, cli_opts, bad_lines)
+            run_app(opts, cli_opts, bad_lines, talk_fd)
     finally:
         glfw_terminate()
         cleanup_ssh_control_masters()
 
 
-def main() -> None:
+
+def main(called_from_panel: bool = False) -> None:
+    global redirected_for_quick_access
     try:
-        _main()
+        if is_macos and launched_by_launch_services and not called_from_panel:
+            with suppress(OSError):
+                os.chdir(os.path.expanduser('~'))
+            if is_quick_access_terminal_app:
+                # we were started by launch services, use the kitten to read
+                # the config and re-run
+                os.execl(kitten_exe(), kitten_exe(), 'quick-access-terminal')
+            set_use_os_log(True)
+        kitty_main(called_from_panel)
     except Exception:
         import traceback
         tb = traceback.format_exc()
         log_error(tb)
         raise SystemExit(1)
+    finally:
+        # we cant rely on this running during module unloading of fast_data_types as Python fails
+        # to unload the module, due to reference cycles, I am guessing.
+        run_at_exit_cleanup_functions()
