@@ -26,6 +26,7 @@ class Launch(RemoteCommand):
     source_window/str: The window to use as source for data or empty string to use active window
     window_title/str: Title for the new window
     cwd/str: Working directory for the new window
+    add_to_session/str: Session name to add created window/tab to
     env/list.str: List of environment variables of the form NAME=VALUE
     var/list.str: List of user variables of the form NAME=VALUE
     os_panel/list.str: List of panel settings
@@ -57,6 +58,7 @@ class Launch(RemoteCommand):
     watcher/list.str: list of paths to watcher files
     bias/float: The bias with which to create the new window in the current layout
     wait_for_child_to_exit/bool: Boolean indicating whether to wait and return child exit code
+    hold_after_ssh/bool: Boolean indicating whether to run a local shell after exiting the ssh session cloned via cwd=current or similar
     '''
 
     short_desc = 'Run an arbitrary process in a new window/tab'
