@@ -24,7 +24,7 @@ from kitty.options.types import Options, defaults
 from kitty.rgb import to_color
 from kitty.types import MouseEvent
 from kitty.utils import read_screen_size
-from kitty.window import decode_cmdline, process_remote_print, process_title_from_child
+from kitty.window import da1, decode_cmdline, process_remote_print, process_title_from_child
 
 
 def parse_bytes(screen, data, dump_callback=None):
@@ -132,10 +132,17 @@ class Callbacks:
         self.last_cmd_cmdline = ''
         self.last_cmd_at = 0
         self.num_of_resize_events = 0
+        self.da1 = []
 
     def on_bell(self) -> None:
         self.bell_count += 1
 
+    def on_da1(self) -> None:
+        payload = da1(get_options())
+        self.da1.append(payload)
+        if self.pty and self.pty.needs_da1:
+            self.pty.send_da1_response(payload)
+
     def on_activity_since_last_focus(self) -> None:
         pass
 
@@ -215,18 +222,24 @@ def filled_history_buf(ynum=5, xnum=5, cursor=Cursor()):
     return ans
 
 
-def retry_on_failure(max_attempts=2, sleep_duration=2):
+is_ci = os.environ.get('CI') == 'true'
+max_attempts = 4 if is_ci else 2
+sleep_duration = 4 if is_ci else 2
+
+
+def retry_on_failure(max_attempts=max_attempts, sleep_duration=sleep_duration):
     def decorator(func):
         @wraps(func)
         def wrapper(*args, **kwargs):
             for attempt in range(max_attempts):
                 try:
                     return func(*args, **kwargs)
-                except Exception as e:
+                except Exception:
                     if attempt < max_attempts - 1: # Don't sleep on the last attempt
                         time.sleep(sleep_duration)
+                        print(f'{func.__name__} failed, retrying in {sleep_duration} seconds', file=sys.stderr)
                     else:
-                        raise e # Re-raise the last exception
+                        raise # Re-raise the last exception
         return wrapper
     return decorator
 
@@ -235,7 +248,7 @@ class BaseTest(TestCase):
 
     ae = TestCase.assertEqual
     maxDiff = 2048
-    is_ci = os.environ.get('CI') == 'true'
+    is_ci = is_ci
 
     def rmtree_ignoring_errors(self, tdir):
         try:
@@ -269,10 +282,13 @@ class BaseTest(TestCase):
 
     def create_pty(
             self, argv=None, cols=80, lines=100, scrollback=100, cell_width=10, cell_height=20,
-            options=None, cwd=None, env=None, stdin_fd=None, stdout_fd=None
+            options=None, cwd=None, env=None, stdin_fd=None, stdout_fd=None, needs_da1=False,
     ):
         self.set_options(options)
-        return PTY(argv, lines, cols, scrollback, cell_width, cell_height, cwd, env, stdin_fd=stdin_fd, stdout_fd=stdout_fd)
+        return PTY(
+            argv, lines, cols, scrollback, cell_width, cell_height, cwd, env, stdin_fd=stdin_fd, stdout_fd=stdout_fd,
+            needs_da1=needs_da1,
+        )
 
     def assertEqualAttributes(self, c1, c2):
         x1, y1, c1.x, c1.y = c1.x, c1.y, 0, 0
@@ -305,7 +321,7 @@ class PTY:
 
     def __init__(
         self, argv=None, rows=25, columns=80, scrollback=100, cell_width=10, cell_height=20,
-        cwd=None, env=None, stdin_fd=None, stdout_fd=None
+        cwd=None, env=None, stdin_fd=None, stdout_fd=None, needs_da1=False,
     ):
         self.is_child = False
         if isinstance(argv, str):
@@ -343,6 +359,7 @@ class PTY:
         self.cell_width = cell_width
         self.cell_height = cell_height
         self.set_window_size(rows=rows, columns=columns)
+        self.needs_da1 = needs_da1
         self.callbacks = Callbacks(self)
         self.screen = Screen(self.callbacks, rows, columns, scrollback, cell_width, cell_height, 0, self.callbacks)
         self.received_bytes = b''
@@ -375,6 +392,9 @@ class PTY:
         if flush:
             self.process_input_from_child(0)
 
+    def send_da1_response(self, data: str) -> None:
+        self.write_to_child('\x1b[' + data, flush=True)
+
     def send_cmd_to_child(self, cmd, flush=False):
         self.callbacks.last_cmd_exit_status = sys.maxsize
         self.last_cmd = cmd
@@ -401,13 +421,13 @@ class PTY:
                 self.process_input_from_child(timeout=end_time - time.monotonic())
             except OSError as e:
                 if not q():
-                    raise Exception(f'Failed to read from pty with error: {e}. Screen contents:  \n {repr(self.screen_contents())}') from e
+                    raise Exception(f'Failed to read from pty with error: {e}. {self.screen_contents_for_error()}') from e
                 return
         if not q():
             msg = 'The condition was not met'
             if timeout_msg is not None:
                 msg = timeout_msg()
-            raise TimeoutError(f'Timed out: {msg}. Screen contents: \n {repr(self.screen_contents())}')
+            raise TimeoutError(f'Timed out after {timeout} seconds: {msg}. {self.screen_contents_for_error()}')
 
     def wait_till_child_exits(self, timeout=30 if BaseTest.is_ci else 10, require_exit_code=None):
         end_time = time.monotonic() + timeout
@@ -419,11 +439,11 @@ class PTY:
                 if require_exit_code is not None and ec != require_exit_code:
                     raise AssertionError(
                         f'Child exited with exit status: {status} code: {ec} != {require_exit_code}.'
-                        f' Screen contents:\n{self.screen_contents()}')
+                        f' {self.screen_contents_for_error()}')
                 return status
             with suppress(OSError):
                 self.process_input_from_child(timeout=0.02)
-        raise AssertionError(f'Child did not exit in {timeout} seconds. Screen contents:\n{self.screen_contents()}')
+        raise AssertionError(f'Child did not exit in {timeout} seconds. {self.screen_contents_for_error()}')
 
     def set_window_size(self, rows=25, columns=80, send_signal=True):
         if hasattr(self, 'screen'):
@@ -434,6 +454,11 @@ class PTY:
             s = struct.pack('HHHH', rows, columns, x_pixels, y_pixels)
             fcntl.ioctl(self.master_fd, termios.TIOCSWINSZ, s)
 
+    def screen_contents_for_error(self):
+        from kitty.window import as_text
+        ans = as_text(self.screen, add_history=True, as_ansi=False)
+        return f'Screen contents as repr:\n{ans!r}\nScreen contents:\n{ans.rstrip()}'
+
     def screen_contents(self):
         lines = []
         for i in range(self.screen.lines):
