@@ -1,15 +1,10 @@
 package disk_cache
 
 import (
-	"crypto/sha256"
-	"encoding/hex"
 	"fmt"
 	"os"
-	"path/filepath"
 	"sync"
 	"time"
-
-	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 var _ = fmt.Print
@@ -34,33 +29,15 @@ type DiskCache struct {
 	entries          Metadata
 	entry_map        map[string]*Entry
 	entries_mod_time time.Time
+	get_dir          string
 }
 
 func NewDiskCache(path string, max_size int64) (dc *DiskCache, err error) {
-	if path, err = filepath.Abs(path); err != nil {
-		return
-	}
-	if err = os.MkdirAll(path, 0o700); err != nil {
-		return
-	}
-	return &DiskCache{Path: path, MaxSize: max_size}, nil
+	return new_disk_cache(path, max_size)
 }
 
 func KeyForPath(path string) (key string, err error) {
-	if path, err = filepath.EvalSymlinks(path); err != nil {
-		return
-	}
-	if path, err = filepath.Abs(path); err != nil {
-		return
-	}
-
-	s, err := os.Stat(path)
-	if err != nil {
-		return
-	}
-	data := fmt.Sprintf("%s\x00%d\x00%d", path, s.Size(), s.ModTime().UnixNano())
-	sum := sha256.Sum256(utils.UnsafeStringToBytes(data))
-	return hex.EncodeToString(sum[:]), nil
+	return key_for_path(path)
 }
 
 func (dc *DiskCache) Get(key string, items ...string) map[string]string {
