@@ -11,8 +11,9 @@ import (
 	"strconv"
 	"strings"
 
-	"kitty/tools/tui/loop"
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 var _ = fmt.Print
@@ -115,6 +116,9 @@ const (
 	GRT_delete_by_cell_zindex // q
 	GRT_free_by_cell_zindex   // Q
 
+	GRT_delete_by_range // r
+	GRT_free_by_range   // R
+
 	GRT_delete_by_column // x
 	GRT_free_by_column   // X
 
@@ -144,6 +148,7 @@ type GraphicsCommand struct {
 
 	z int32
 
+	DisableCompression       bool
 	WrapPrefix, WrapSuffix   string
 	EncodeSerializedDataFunc func(string) string
 
@@ -199,6 +204,9 @@ func (self GraphicsCommand) String() string {
 	return ans + ")"
 }
 
+var debugprintln = tty.DebugPrintln
+var _ = debugprintln
+
 func (self *GraphicsCommand) serialize_to(buf io.StringWriter, chunk string) (err error) {
 	var ws func(string)
 	if self.EncodeSerializedDataFunc == nil {
@@ -246,7 +254,7 @@ func compress_with_zlib(data []byte) []byte {
 	var b bytes.Buffer
 	b.Grow(len(data) + 128)
 	w := zlib.NewWriter(&b)
-	w.Write(data)
+	_, _ = w.Write(data)
 	w.Close()
 	return b.Bytes()
 }
@@ -254,7 +262,7 @@ func compress_with_zlib(data []byte) []byte {
 func (self *GraphicsCommand) AsAPC(payload []byte) string {
 	buf := strings.Builder{}
 	buf.Grow(1024)
-	self.WriteWithPayloadTo(&buf, payload)
+	_ = self.WriteWithPayloadTo(&buf, payload)
 	return buf.String()
 }
 
@@ -264,22 +272,23 @@ func (self *GraphicsCommand) WriteWithPayloadTo(o io.StringWriter, payload []byt
 		return self.serialize_to(o, "")
 	}
 	if len(payload) <= compression_threshold {
-		return self.serialize_to(o, base64.StdEncoding.EncodeToString(payload))
+		return self.serialize_to(o, base64.RawStdEncoding.EncodeToString(payload))
 	}
 	gc := *self
-	if self.Format() != GRT_format_png {
+	if !self.DisableCompression && self.Format() != GRT_format_png {
 		compressed := compress_with_zlib(payload)
 		if len(compressed) < len(payload) {
 			gc.SetCompression(GRT_compression_zlib)
 			payload = compressed
 		}
 	}
-	data := base64.StdEncoding.EncodeToString(payload)
+	const chunk_size = 128 * 1024
+	data := base64.RawStdEncoding.EncodeToString(payload)
 	for len(data) > 0 && err == nil {
 		chunk := data
-		if len(data) > 4096 {
-			chunk = data[:4096]
-			data = data[4096:]
+		if len(data) > chunk_size {
+			chunk = data[:chunk_size]
+			data = data[chunk_size:]
 		} else {
 			data = ""
 		}
@@ -417,7 +426,7 @@ func GraphicsCommandFromAPCPayload(raw []byte) *GraphicsCommand {
 	var gc GraphicsCommand
 
 	add_key := func(pos int) {
-		gc.SetString(current_key, utils.UnsafeBytesToString(raw[value_start_at:pos]))
+		_ = gc.SetString(current_key, utils.UnsafeBytesToString(raw[value_start_at:pos]))
 	}
 
 	for pos, ch := range raw {
