@@ -7,15 +7,20 @@ import (
 	"encoding/base32"
 	"fmt"
 	"io/fs"
-	not_rand "math/rand"
+	not_rand "math/rand/v2"
 	"os"
 	"os/exec"
 	"os/user"
 	"path/filepath"
 	"runtime"
+	"sort"
 	"strconv"
 	"strings"
+	"sync"
+	"syscall"
+	"unicode/utf8"
 
+	"github.com/shirou/gopsutil/v3/process"
 	"golang.org/x/sys/unix"
 )
 
@@ -62,9 +67,17 @@ func Abspath(path string) string {
 	return path
 }
 
-var KittyExe = Once(func() string {
-	exe, err := os.Executable()
-	if err == nil {
+var KittyExe = sync.OnceValue(func() string {
+	if kitty_pid := os.Getenv("KITTY_PID"); kitty_pid != "" {
+		if kp, err := strconv.ParseInt(kitty_pid, 10, 32); err == nil {
+			if p, err := process.NewProcess(int32(kp)); err == nil {
+				if exe, err := p.Exe(); err == nil && filepath.IsAbs(exe) && filepath.Base(exe) == "kitty" {
+					return exe
+				}
+			}
+		}
+	}
+	if exe, err := os.Executable(); err == nil {
 		ans := filepath.Join(filepath.Dir(exe), "kitty")
 		if s, err := os.Stat(ans); err == nil && !s.IsDir() {
 			return ans
@@ -117,11 +130,11 @@ func ConfigDirForName(name string) (config_dir string) {
 	return
 }
 
-var ConfigDir = Once(func() (config_dir string) {
+var ConfigDir = sync.OnceValue(func() (config_dir string) {
 	return ConfigDirForName("kitty.conf")
 })
 
-var CacheDir = Once(func() (cache_dir string) {
+var CacheDir = sync.OnceValue(func() (cache_dir string) {
 	candidate := ""
 	if edir := os.Getenv("KITTY_CACHE_DIRECTORY"); edir != "" {
 		candidate = Abspath(Expanduser(edir))
@@ -134,7 +147,7 @@ var CacheDir = Once(func() (cache_dir string) {
 		}
 		candidate = filepath.Join(Expanduser(candidate), "kitty")
 	}
-	os.MkdirAll(candidate, 0o755)
+	_ = os.MkdirAll(candidate, 0o755)
 	return candidate
 })
 
@@ -151,7 +164,7 @@ func macos_user_cache_dir() string {
 		if err != nil {
 			return false
 		}
-		stat, ok := s.Sys().(unix.Stat_t)
+		stat, ok := s.Sys().(syscall.Stat_t)
 		return ok && s.IsDir() && int(stat.Uid) == os.Geteuid() && s.Mode().Perm() == 0o700 && unix.Access(m, unix.X_OK|unix.W_OK|unix.R_OK) == nil
 	}
 
@@ -176,7 +189,7 @@ func macos_user_cache_dir() string {
 	return ""
 }
 
-var RuntimeDir = Once(func() (runtime_dir string) {
+var RuntimeDir = sync.OnceValue(func() (runtime_dir string) {
 	var candidate string
 	if q := os.Getenv("KITTY_RUNTIME_DIRECTORY"); q != "" {
 		candidate = q
@@ -291,3 +304,29 @@ func ResolveConfPath(path string) string {
 	}
 	return cs
 }
+
+// Longest common path. Must be passed paths that have been cleaned by filepath.Clean
+func Commonpath(paths ...string) (longest_prefix string) {
+	switch len(paths) {
+	case 0:
+		return
+	case 1:
+		return paths[0]
+	default:
+		sort.Strings(paths)
+		a, b := paths[0], paths[len(paths)-1]
+		sz := 0
+		for a != "" && b != "" {
+			ra, na := utf8.DecodeRuneInString(a)
+			rb, nb := utf8.DecodeRuneInString(b)
+			if ra != rb {
+				break
+			}
+			sz += na
+			a = a[na:]
+			b = b[nb:]
+		}
+		longest_prefix = paths[0][:sz]
+	}
+	return
+}
